# ------------------------------------------------------------
# alert_tools_install.sh
#
# Used to install (or upgrade) Mark Dickinsons Alert Tools utilities.
# Binary and source releases.
#
# Installs as either an agent or a server,
#   if an agent
#      customises supplied files with ipaddr and port of remote collector
#   if a server/collector
#      customises supplied files for current host
#      created rcx.d scripts if required
#
# Functions...
#    Get installation parameters
#    Untar the files into the target dir
#    If a source release do compiles now
#    Customise supplied files with sed to reflect install options
#    If a server release create rcx.d links if user wants them
# 
# ------------------------------------------------------------
#
#

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#          Create some default values.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
sourcedir=`pwd`
targetdir="/opt/dickinson/alert_management"
myuserid=`whoami`
targetuid=${myuserid}
targetgroup="root"
userc="n"
rcname="marks_alert_collector"
testvar="x"
portnum="9003"
server_started="FALSE"
installed_rc_script="N"
installtype="SERVER"
ipaddrtouse="ALL"
hostnametouse=`hostname`
workfile="/tmp/alert_tools_install"

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# set testvar to either y or n, enforce one of those.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
testvar_to_yn() {
   testvar2="x"
   while [ "${testvar2}." != "z." ];
   do
      read -p "Reply y or n ? " testvar
	  if [ "${testvar}." = "y." ];
	  then
			  testvar2="z"
	  fi
	  if [ "${testvar}." = "n." ];
	  then
			  testvar2="z"
	  fi
	  echo "Got ${testvar}"
   done
} # testvar_to_yn

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# As we package the files under my development userid
# and group, when they are untarred the groupid is
# inlikely to match one that exists on the system.
# To avoid orphaned group numbers on the file we want
# to set the group ownership to the group of the user
# selected for the install.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
set_groupid() {
  aa=`grep "${targetuid}:" /etc/passwd | cut -d: -f4 | head -1`
  targetgroup=`grep ":${aa}:" /etc/group | cut -d: -f1 | head -1`
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#        Show the disclaimer.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
disclaimer() {
   temp_fname="/tmp/ac_install.tmp"
   clear
   echo "Centralised Alert Console Application, alert collection and forwarding tools" > ${temp_fname}
   echo "Copyright (C) 2003    Mark Dickinson,  All rights reserved." >> ${temp_fname}
   echo " " >> ${temp_fname}
   echo "           DISCLAIMER ! - READ AND ACCEPT TO CONTINUE" >> ${temp_fname}
   echo "" >> ${temp_fname}
   echo "This software is not warrented for suitability in any installation" >> ${temp_fname}
   echo "other than my own, use it at your own risk." >> ${temp_fname}
   echo "" >> ${temp_fname}
   echo "This program is distributed in the hope that it will be useful," >> ${temp_fname}
   echo "but WITHOUT ANY WARRANTY; without even the implied warranty of" >> ${temp_fname}
   echo "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." >> ${temp_fname}
   echo "" >> ${temp_fname}
   echo "BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY" >> ${temp_fname}
   echo "FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN" >> ${temp_fname}
   echo "OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES" >> ${temp_fname}
   echo "PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED" >> ${temp_fname}
   echo "OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF" >> ${temp_fname}
   echo "MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS" >> ${temp_fname}
   echo "TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE" >> ${temp_fname}
   echo "PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING," >> ${temp_fname}
   echo "REPAIR OR CORRECTION." >> ${temp_fname}
   echo "" >> ${temp_fname}
   echo "IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING" >> ${temp_fname}
   echo "WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING" >> ${temp_fname}
   echo "ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT" >> ${temp_fname}
   echo "OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED" >> ${temp_fname}
   echo "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY" >> ${temp_fname}
   echo "YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER" >> ${temp_fname}
   echo "PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE" >> ${temp_fname}
   echo "POSSIBILITY OF SUCH DAMAGES." >> ${temp_fname}
   echo "" >> ${temp_fname}
   echo "--- Do you accept these terms and conditions, reply y or n ---" >> ${temp_fname}
   more ${temp_fname}
   rm -f ${temp_fname}
   testvar="x"
   testvar_to_yn
   if [ "${testvar}." != "y." ];
   then
       echo " "
       echo "Installation cancelled by user !"
       exit 1
   fi
} # disclaimer

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#        Show the install intoduction banner.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
intro() {
   clear
   echo " "
   echo "*************************************************"
   echo "*  Installation script for marks alert tools    *"
   echo "*************************************************"
   echo " "
   echo "There are two types of possible installs. A SERVER"
   echo "install, which allows you to configure and start"
   echo "the install, or a AGENT install which configures"
   echo "the installation to forward alerts to a previously"
   echo "installed server installation."
   echo " "
   echo "The ful binary set will be installed with each"
   echo "type, but the customisation of each is different."
   echo " "
   echo "Which type of installation are you doing, the"
   echo "(s)ERVER or (a)GENT installation ?."
   echo " "
   testvar="x"
   while [ "${testvar}." = "x." ];
   do
       read -p "Installation type (s/a) ?" testvar
       if [ "${testvar}." != "s." ];
       then
          if [ "${testvar}." != "a." ];
          then
             testvar="x"
          fi
       fi
   done
   if [ "${testvar}." != "s." ];
   then
      installtype="SERVER"
   else
      installtype="AGENT"
   fi
} # intro

intro_server() {
   echo "*************************************************"
   echo "*  Installation script for marks alert tools    *"
   echo "*             SERVER Installation               *"
   echo "*************************************************"
   echo "Before beginning you need to know..."
   echo "  The directory you are installing to"
   echo "  The user that will own the installed files"
   echo "  Decided on  TcpIp port number to use (default is 9003)"
   echo "  Whether you are binding on one interface or all, if"
   echo "  you are binding on a single interface you must know "
   echo " the tcpip address of the interface."
   echo " "
   echo "If this is an upgrade ensure your existing server is"
   echo "SHUTDOWN before proceeding."
   read -p "Continue (y/n) ?" testvar
   if [ "${testvar}." != "y."];
   then
     echo "Install aborted by user"
     exit 1
   fi
} # intro_server

show_parms_server() {
   clear
   echo "Enter the number to change a parameter value,"
   echo "     or 'i' to begin the installation"
   echo "          [use ctrl-c to abort]"
   echo " "
   echo "If you change the default port number refer to the file"
   echo "USER_MANUAL.txt to see how to connect to the port as"
   echo "all programs are compiled to use 9003 by default."
   echo " "
   echo "1) Target directory : ${targetdir}"
   echo "2) Files are to be owned by : ${targetuid}"
   echo "3) TcpIp port number to use : ${portnum}"
   echo "4) The ip-address on this server to bind to,"
   echo "   if not provided all interfaces will be"
   echo "   listened on : ${ipaddrtouse}"
   echo " "
} # show_parms_server

get_parms_server() {
   testvar="x"
   while [ "${testvar}." != "i." ];
   do
      show_parms_server
	  read -p "Change which option (or 'i' to install with above): " testvar
	  if [ "${testvar}." = "1." ];
	  then
			  read -r -p "New target directory ? " targetdir
			  if [ "${targetdir}." = "." ];
			  then
					  targetdir="/opt/dickinson/alert_management"
					  echo "You must provide one."
			  fi
	  fi
	  if [ "${testvar}." = "2." ];
	  then
			  read -p "Which user will own the files ? " targetuid
			  if [ "${targetuid}." = "." ];
			  then
					  targetuid=${myuserid}
					  echo "You must provide one."
			  fi
	  fi
	  if [ "${testvar}." = "3." ];
	  then
			  read -p "Select port number to use (5000+), recomended is 9003 ?" portnum
			  if [ "${portnum}." = "." ];
			  then
					  portnum="9003"
					  echo "You must provide one."
			  fi
	  fi
	  if [ "${testvar}." = "4." ];
	  then
			  read -p "Select ip-address to bind to ?" ipaddrtouse
			  if [ "${ipaddrtouse}." = "." ];
			  then
					  ipaddrtouse="ALL"
					  echo "You must provide one."
			  else
					  # make sure it's valid for this system
                      zz=`ifconfig -a | grep "${ipaddrtouse}" | grep -v "grep"`
					  if [ "${zz}." = "." ];
					  then
					     echo "IP-ADDRESS ${ipaddrtouse} is not a valid interface for this system"
					     echo "I suggest using 'ifconfig -a' to get a legal address."
					     ipaddrtouse="ALL"
					  fi
			  fi
	  fi
   done
} # get_parms_server

intro_agent() {
   echo "*************************************************"
   echo "*  Installation script for marks alert tools    *"
   echo "*             AGENT Installation                *"
   echo "*************************************************"
   echo "Before beginning you need to know..."
   echo "  The directory you are installing to"
   echo "  The user that will own the installed files"
   echo "  The port number the server system uses (default is 9003)"
   echo "  The ip address the server system listens on."
   echo " "
   echo "If this is an upgrade ensure all alert forwarding"
   echo "scripts are stopped before proceeding."
   read -p "Continue (y/n) ?" testvar
   if [ "${testvar}." != "y."];
   then
     echo "Install aborted by user"
     exit 1
   fi
} # intro_agent

show_parms_agent() {
   clear
   echo "Enter the number to change a parameter value,"
   echo "     or 'i' to begin the installation"
   echo "          [use ctrl-c to abort]"
   echo " "
   echo "If you change the default port number refer to the file"
   echo "USER_MANUAL.txt to see how to connect to the port as"
   echo "all programs are compiled to use 9003 by default."
   echo " "
   echo "1) Target directory : ${targetdir}"
   echo "2) Files are to be owned by : ${targetuid}"
   echo "3) TcpIp port number used by the remote server : ${portnum}"
   echo "4) The ip-address the remote server listens to : ${ipaddrtouse}"
   echo "5) The hostname of the remote server : ${hostnametouse}"
   echo " "
} # show_parms_agent

get_parms_agent() {
   testvar="x"
   while [ "${testvar}." != "i." ];
   do
      show_parms_agent
	  read -p "Change which option (or 'i' to install with above): " testvar
	  if [ "${testvar}." = "1." ];
	  then
			  read -r -p "New target directory ? " targetdir
			  if [ "${targetdir}." = "." ];
			  then
					  targetdir="/opt/dickinson/alert_management"
					  echo "You must provide one."
			  fi
	  fi
	  if [ "${testvar}." = "2." ];
	  then
			  read -p "Which user will own the files ? " targetuid
			  if [ "${targetuid}." = "." ];
			  then
					  targetuid=${myuserid}
					  echo "You must provide one."
			  fi
	  fi
	  if [ "${testvar}." = "3." ];
	  then
			  read -p "What port number is the remote server using (default 9003) ?" portnum
			  if [ "${portnum}." = "." ];
			  then
					  portnum="9003"
					  echo "You must provide one."
			  fi
	  fi
	  if [ "${testvar}." = "4." ];
	  then
			  read -p "What is the ip-address the remote server listens on ?" ipaddrtouse
			  if [ "${ipaddrtouse}." = "." ];
			  then
					  ipaddrtouse=""
					  echo "You must provide one."
			  else
					  # make sure it's valid for this system
                      echo "Wait, pinging for confirmation..."
                      zz=`ping -c1 ${ipaddrtouse} | grep -i "unreachable"`
					  if [ "${zz}." != "." ];
					  then
					     echo "IP-ADDRESS ${ipaddrtouse} is not responding, cannot be used"
					     ipaddrtouse=""
                         read -p "Depress ENTER to continue" testvar
					  fi
			  fi
	  fi
	  if [ "${testvar}." = "5." ];
	  then
			  read -p "What is the hostname of the remote server ?" hostnametouse
			  if [ "${hostnametouse}." = "." ];
			  then
					  hostnametouse=`hostname`
					  echo "You must provide one."
			  else
					  # make sure it's valid for this system
                      echo "Wait, pinging for confirmation..."
                      zz=`ping -c1 ${hostnametouse} | grep -i "unreachable"`
					  if [ "${zz}." != "." ];
					  then
					     echo "Host named ${hostnametouse} is not responding, cannot be used"
					     hostnametouse=`hostname`
                         read -p "Depress ENTER to continue" testvar
					  fi
			  fi
	  fi
   done
} # get_parms_agent

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create the target directory, untar the shipped tar
# file into it, create required subdirectories, set the
# file and directory permissions as needed.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
install_appl_files() {
  if [ ! -d ${targetdir} ];
  then
		  mkdir -p ${targetdir}
		  chown ${targetuid}:${targetgroup} ${targetdir}
		  chmod 755 ${targetdir}
  fi
  if [ ! -d ${targetdir} ];
  then
		  echo "Unable to create directory ${targetdir}, aborting !"
		  exit 1
  fi
  if [ "${targetdir}." != "${sourcedir}." ];
  then
     cd ${targetdir}
  fi
  tar -xvf ${sourcedir}/alert_tools_collection.tar
  if [ $? -ne 0 ];
  then
     echo "Unable to untar the files, aborting !"
	 exit 1;
   fi
   # Added this block specifically for source releases.
   # If a source only release, compile the binaries we need
   if [ -d source ];    # directory only present in source release
   then
      echo "This is a source release. Compiling modules..."
      cd source
      ./compile_all.sh
      xx=$?
      if [ "${xx}." != "0." ];
      then
         echo "There are problems with the compile on this system !"
         echo "Aborting installation."
         exit 1
      fi
      # else it worked, move the binaries where they should be and clean up
      mv temp_bin/* ${targetdir}
      rm -rf temp_bin
	  cd ..
   fi
   chown -R ${targetuid}:${targetgroup} *
} # install_appl_files

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create the required startup and shutdown scripts for
# the job scheduler. We want it to start in runlevel
# 3 when the system is multiuser. We need the K links
# as well to ensure the scheduler is shutdown cleanly
# when the system is.
# Used only by a SERVER install.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
install_rc_scripts() {
   echo " "
   echo "Do you wish to have the startup and files placed"
   echo "in the /etc/rc.d directories so the server starts"
   echo "automatically ?"
   echo " "
   testvar="x"
   testvar_to_yn
   if [ "${testvar}." = "y." ];
   then
      # ensure we don't overwrite anything
      while [ -f /etc/init.d/${rcname} ];
	  do
          echo "/etc/init.d/${rcname} exists, select a new name or 'q' to skip"
          read -p "New file name ? " testvar
          if [ "${testvar}." = "q." ];
          then
             return
          fi
          if [ "${testvar}." != "." ];
          then
             rcname=${testvar}
          fi
	  done
      # setup
      echo "Copying rc script to /etc/rc.d/init.d/${rcname}"
      cp -pi ${targetdir}/scripts/start_alertserver.sh /etc/rc.d/init.d/${rcname}
	  chmod 700 /etc/rc.d/init.d/${rcname}
	  chown ${targetuid}:${targetgroup} /etc/rc.d/init.d/${rcname}
      echo "Creating symbolic links in /etc/rc.d directories"
	  echo "   /etc/rc.d/rc0.d - K12_${rcname}"
      cd /etc/rc.d/rc0.d
      ln -s ../init.d/${rcname} K12_${rcname}
	  echo "   /etc/rc.d/rc1.d - K12_${rcname}"
      cd /etc/rc.d/rc1.d
      ln -s ../init.d/${rcname} K12_${rcname}
	  echo "   /etc/rc.d/rc6.d - K12_${rcname}"
      cd /etc/rc.d/rc6.d
      ln -s ../init.d/${rcname} K12_${rcname}
	  echo "   /etc/rc.d/rc3.d - S70_${rcname}"
      cd /etc/rc.d/rc3.d
      ln -s ../init.d/${rcname} S70_${rcname}
      installed_rc_script="Y"
   fi
} # install_rc_scripts

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Start the server. Where we start it from depends on
# whether the rc.d files were created or not.
# Used only by a SERVER install.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
start_server() {
  echo " "
  echo "Do you wish to start the server now ?"
  testvar="x"
  testvar_to_yn
  if [ "${testvar}." = "y." ];
  then
    server_started="TRUE";
    if [ "${installed_rc_script}" = "Y" ];
    then
       if [ ! -x /etc/rc.d/init.d/${rcname} ];
       then
          echo "The installation to /etc/rc.d/init.d appears to have failed !"
          echo "Can't start scheduler, /etc/rc.d/init.d/${rcname} not executable"
        else
           echo "Issuing command: /etc/rc.d/init.d/${rcname} start"
           /etc/rc.d/init.d/${rcname} start
        fi
     else
        echo "Issuing command: ${targetdir}/scripts/start_alertserver start"
        ${targetdir}/scripts/start_alertserver start
    fi
  fi
} # start_server 

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Customise the init.d scripts and the sample job scripts
# for this installation.
# Used by both install types.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
customise_scripts() {
   cd ${targetdir}
   echo "Customising startup scripts for directory ${targetdir}..."

   # The startup script for the server
   if [ -f scripts/start_alertserver.sh ];
   then
      echo "s'/opt/dickinson/alert_management'${targetdir}'g" > ${workfile}
      echo "s'userid=\"mark\"'userid=\"${targetuid}\"'g" >> ${workfile}
      cat ${targetdir}/scripts/start_alertserver.sh | sed -f ${workfile} > /tmp/delme
      mv /tmp/delme ${targetdir}/scripts/start_alertserver.sh
      chmod 700 ${targetdir}/scripts/start_alertserver.sh
      chown ${targetuid}:${targetgroup} ${targetdir}/scripts/start_alertserver.sh
      cat ${targetdir}/scripts/start_alertserver.sh | sed -f ${workfile} > /tmp/delme
      mv /tmp/delme ${targetdir}/scripts/start_alertserver.sh
      chmod 700 ${targetdir}/scripts/start_alertserver.sh
      chown ${targetuid}:${targetgroup} ${targetdir}/scripts/start_alertserver.sh
   else
           echo "Unable to find file scripts/start_alertserver.sh, aborting !"
		   exit 1
   fi

   # And the customised one
   if [ -f scripts/start_server_and_tailtask.sh ];
   then
      echo "s'/opt/dickinson/alert_management'${targetdir}'g" > ${workfile}
      echo "s'userid=\"mark\"'userid=\"${targetuid}\"'g" >> ${workfile}
      cat ${targetdir}/scripts/start_server_and_tailtask.sh | sed -f ${workfile} > /tmp/delme
      mv /tmp/delme ${targetdir}/scripts/start_server_and_tailtask.sh
      chmod 700 ${targetdir}/scripts/start_server_and_tailtask.sh
      chown ${targetuid}:${targetgroup} ${targetdir}/scripts/start_server_and_tailtask.sh
      cat ${targetdir}/scripts/start_server_and_tailtask.sh | sed -f ${workfile} > /tmp/delme
      mv /tmp/delme ${targetdir}/scripts/start_server_and_tailtask.sh
      chmod 700 ${targetdir}/scripts/start_server_and_tailtask.sh
      chown ${targetuid}:${targetgroup} ${targetdir}/scripts/start_server_and_tailtask.sh
   fi

   # And the critical one, the config file used by all scripts
   rm -f ${workfile}
   if [ -f ${targetdir}/etc/config.txt ];
   then
      if [ "${portnum}." != "9003" ];
      then
         echo "s'PORT=9003'PORT=${portnum}'g" >> ${workfile}
      fi
      if [ "${ipaddrtouse}." != "ALL" ];
      then
         echo "s'SERVER_IPADDR=127.0.0.1'SERVER_IPADDR=${ipaddrtouse}'g" >> ${workfile}
      fi
      echo "s'SERVER_NAME=falcon'SERVER_NAME=${hostnametouse}'g" >> ${workfile}
      echo "s'/opt/dickinson/alert_management'${targetdir}'g" >> ${workfile}
      cat ${targetdir}/etc/config.txt | sed -f ${workfile} > /tmp/delme
      mv /tmp/delme ${targetdir}/etc/config.txt
      chmod 644 ${targetdir}/etc/config.txt
      chown ${targetuid}:${targetgroup} ${targetdir}/etc/config.txt
      rm -f ${workfile}
   else
           echo "Unable to find file etc/config.txt, aborting !"
		   exit 1
   fi
} # customise_scripts

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# When we have finished, show the appropriate start
# command for the server based on whether the rc script
# was installed or not.
# Used only by a SERVER install.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
show_start_command() {
    if [ "${installed_rc_script}" = "Y" ];
    then
       echo "The alert collector server will be started at each reboot..."
       echo "    '/etc/rc.d/init.d/${rcname} start' may be used for manual starts"
    else
       echo "The alert collector server to be manually restarted when needed..."
       echo "  use '${targetdir}/scripts/start_alertserver start'"
    fi
} # show_start_command

create_uninstaller() {
   echo "WAIT !, creating uninstall script for you."
   thename="uninstall_dickinson_alert_tools.sh"
   cd ${targetdir}
   echo "#!/bin/bash" > ${thename}
   echo "# -------------------------------------------------" >> ${thename}
   echo "# Generated script." >> ${thename}
   echo "# This can be used to uninstall the scheduler." >> ${thename}
   echo "# -------------------------------------------------" >> ${thename}
   echo "myuserid=\`whoami\`" >> ${thename}
   echo "if [ ${myuserid} != \"root\" ];" >> ${thename}
   echo "then" >> ${thename}
   echo "  echo \"You must be the root user to run this script\" " >> ${thename}
   echo "  exit 1" >> ${thename}
   echo "fi" >> ${thename}
   echo "#" >> ${thename}

   echo "# If we are not running from /tmp do so now." >> ${thename}
   echo 'mypath=`dirname $0`' >> ${thename}
   echo 'if [ "${mypath}." != "/tmp." ];' >> ${thename}
   echo "then" >> ${thename}
   echo '   cp $0 /tmp/uninstall_dickinson_alert_tools.sh' >> ${thename}
   echo "   chmod 766 /tmp/uninstall_dickinson_alert_tools.sh" >> ${thename}
   echo "   /tmp/uninstall_dickinson_alert_tools.sh" >> ${thename}
   echo "   exit 0" >> ${thename}
   echo "fi" >> ${thename}

   echo "#" >> ${thename}
   echo "# Confirm the removal first" >> ${thename}
   echo "echo \"This will remove all mark dickinsons alert tools from your system\" " >> ${thename}
   echo "read -p \"Is this what you wish to do (y/n) ?\" testvar" >> ${thename}
   echo "if [ \"\${testvar}.\" != \"y.\" ];" >> ${thename}
   echo "then" >> ${thename}
   echo "   echo \"Cancelled !\" " >> ${thename}
   echo "   exit 1" >> ${thename}
   echo "fi" >> ${thename}
   echo "#" >> ${thename}
   echo "#" >> ${thename}
   if [ ${installed_rc_script} = "Y" ];
   then
      echo "# Remove symbolic links the install created." >> ${thename}
      echo "if [ -h /etc/rc.d/rc0.d/K10_${rcname} ];" >> ${thename}
      echo "then"  >> ${thename}
      echo "   rm -f /etc/rc.d/rc0.d/K10_${rcname}"  >> ${thename}
      echo "   echo \"deleted /etc/rc.d/rc0.d/K10_${rcname}\" "  >> ${thename}
      echo "fi"  >> ${thename}
      echo "if [ -h /etc/rc.d/rc1.d/K10_${rcname} ];"  >> ${thename}
      echo "then"  >> ${thename}
      echo "   rm -f /etc/rc.d/rc1.d/K10_${rcname}"  >> ${thename}
      echo "   echo \"deleted /etc/rc.d/rc1.d/K10_${rcname}\" "  >> ${thename}
      echo "fi"  >> ${thename}
      echo "if [ -h /etc/rc.d/rc6.d/K10_${rcname} ];"  >> ${thename}
      echo "then"  >> ${thename}
      echo "   rm -f /etc/rc.d/rc6.d/K10_${rcname}"  >> ${thename}
      echo "   echo \"deleted /etc/rc.d/rc6.d/K10_${rcname}\" "  >> ${thename}
      echo "fi"  >> ${thename}
      echo "if [ -h /etc/rc.d/rc3.d/S99_${rcname} ];"  >> ${thename}
      echo "then"  >> ${thename}
      echo "   rm -f /etc/rc.d/rc3.d/S99_${rcname}"  >> ${thename}
      echo "   echo \"deleted /etc/rc.d/rc3.d/S99_${rcname}\" "  >> ${thename}
      echo "fi"  >> ${thename}
      echo "if [ -f /etc/rc.d/init.d/${rcname} ];"  >> ${thename}
      echo "then"  >> ${thename}
      echo "   rm -f /etc/rc.d/init.d/${rcname}"  >> ${thename}
      echo "   echo \"deleted /etc/rc.d/init.d/${rcname}\" "  >> ${thename}
      echo "fi"  >> ${thename}
   else
      echo "# No /etc/rc.d scripts were created at install, none removed"  >> ${thename}
   fi
   echo "#"  >> ${thename}
   echo "# We assume the user installed this application into a unique"  >> ${thename}
   echo "# directory, but we get confirmation to be sure..."  >> ${thename}
   echo "echo \" \" " >> ${thename}
   echo "echo \" \" " >> ${thename}
   echo "echo \"**WARNING** All files under ${targetdir} will be deleted\" " >> ${thename}
   echo "read -p \"Is that what you intend (y/n) ?\" testvar" >> ${thename}
   echo "if [ \"\${testvar}.\" = \"y.\" ];" >> ${thename}
   echo "then" >> ${thename}
   echo "  rm -fr ${targetdir}" >> ${thename}
   echo "fi" >> ${thename}
   echo "# " >> ${thename}
   echo "echo \"Uninstall completed. Remove any components you may have\" " >> ${thename}
   echo "echo \"customised as appropriate for your installation.\" " >> ${thename}
   echo "exit 0" >> ${thename}
   echo " "
   chmod 700 ${targetdir}/${thename}
   echo "*****************************************************************"
   echo "An uninstall script based on your configuration has been saved in"
   echo "    ${targetdir}/${thename}"
   echo "*****************************************************************"
   echo " "
} # create_uninstaller

# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# MAINLINE: Call the procedures so carefully defined
#           above to actually do this installation.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# - - - - - - - - - - - - - - - - - - - - - - - - - - - -
if [ "${myuserid}." = "root." ];
then
   disclaimer
   intro
   if [ "${installtype}." = "SERVER" ];
   then
      intro_server
      get_parms_server
      set_groupid
      install_appl_files
      customise_scripts
      install_rc_scripts
      start_server
      show_start_command
   else # AGENT
      ipaddrtouse="" # installer must supply, don't default
      intro_agent 
      get_parms_agent 
      set_groupid
      install_appl_files
      customise_scripts
   fi
   read -p "Depress ENTER to continue" testvar
   create_uninstaller
   echo "Refer to the file ${targetdir}/USER_MANUAL.txt for useage instructions"
else
  echo "You must be root to run $0"
  echo "Login as root then rerun this script."
fi

# Finished
exit 0
