#!/bin/bash
# ===========================================================================
# is_automated:
#
# Check the script and subject passed to us against the automation table
# to see if we can automate this event.
#
# We don't just see if the entry exist, we also validate all the requirements
# of the rule in this check.
#
# We return the text string "YES" or "NO" to indicate if the event can be
# automated. If the NO is because of an error we exit with 1 if the caller
# needs to handle it, 2 if we have written an error message ourselves but
# the caller may want to do more processing. If there is a valid automation
# rule we exit 0.
#
# Bugfix: exporting the key variables didn't work, caller couldn't see them.
#         I pass then back with the yes string for the caller to parse out now.
# ===========================================================================
mydir=`dirname $0`
myname=`basename $0`
scriptname="$1"
subjectname="$2"

# We need the tools_library here
if [ ! -f ${mydir}/tools_library ];
then
   echo "NO"
   logger "${myname}: Missing ${mydir}/tools_library file"
   exit 1
fi
. ${mydir}/tools_library

# Does an automation_table exist, if not then no automation for event
if [ ! -f ${mydir}/automation_table ];
then
   echo "NO"
   raise_critical_alert "AUTOMATIONCHK" "${myname}: Missing ${mydir}/automation_table file"
   exit 2
fi

# Does  a matching script/subject exist in the automation_table, if not then no automation for event
# Note: use spaces to ensure we don't get a part of a name triggering a match
# ie: don't trigger on a_test_file_name while looking for a_test_file.
testvar=`cat ${mydir}/automation_table | grep -v "#" | grep "${scriptname} " | grep " ${subjectname} "`
if [ "${testvar}." = "." ];
then
   echo "NO"
   exit 0
fi

# Get the automation key, we can use it in messages from here
testkey=`echo "${testvar}" | awk {'print $1'}`

# Does the configured automation script exist, if not no automation and log an error
testfile=`echo "${testvar}" | awk {'print $5'}`
if [ ! -x ${mydir}/scripts/${testfile} ];
then
   echo "NO"
   raise_critical_alert "${testkey}" "${myname}: Missing automation script ${mydir}/scripts/${testfile}"
   exit 2
fi

# Are we running as the correct user to do the automation, if not no automation
testuser=`echo "${testvar}" | awk {'print $4'}`
# Batch jobs don't set this by default
if [ "${LOGNAME}." = "." ];
then
   LOGNAME=`whoami`
fi
if [ "${testuser}." != "${LOGNAME}." ]
then
   echo "NO"
   raise_critical_alert "${testkey}" "${scriptname}: Automation needs ${testuser}, current='${LOGNAME}'"
   exit 2
fi

# All checks passed, can be automated, export key variables we checked and say YES
AUTOMATION_SCRIPT="${mydir}/scripts/${testfile}"
AUTOMATION_KEY="${testkey}"
# export AUTOMATION_SCRIPT AUTOMATION_KEY
# export did't work, pass back with string
echo "YES ${AUTOMATION_KEY} ${AUTOMATION_SCRIPT}"
exit 0
