#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------

# -----------------------------------------------------------
#
#   monitor_var_log_control.sh
#
#   provide a single point of control to the task that tails
#   and parses the /var/log/messages file.
#
#    requires:
#       log_parser
#       samples/log_parser.sample_config
#       samples/log_parser_processor.sh
#       scripts/raise_alert.sh
#       scripts/end_alert.sh
#
# For linux we use a combination of GNU tail program flags to let tail
# itself detect when logs are rolled and open the new one. For other servers
# you will need to add a stop/start of the tail task into the script that
# rolls your log files. This is in the sample monitor_var_log_control.sh
# script.
 
scriptpath=`dirname $0`
myname=`basename $0`
mypath=`pwd`
RUNPID_DIR="${scriptpath}/../run"
cd ${scriptpath}   # all files relevant to samples directory.

# Check the variables we require are available, and use those to check that
# the directories and files we require are available. They should be
# relative to the script path.
check_control_path() {
   # load the global cariables we need
   if [ -f ../etc/config.txt ];
   then
      . ../etc/config.txt
   else
      logger "*FATAL* ${myname}: No configuration files are available in ${scriptpath}/../etc !"
      exit 1
   fi
   # if the pid directory does not exist yet, create it.
   if [ ! -d ${RUNPID_DIR} ];
   then
      mkdir ${RUNPID_DIR}
      chmod 775 ${RUNPID_DIR}
   fi
} # end check_control_path


#   issues a background command to start tailing the
#   log /var/adm/messages, pipes to the log_parser which
#   will process the messages based on the default rule
#   file provided. Any output from rule matches is piped
#   to the script log_parser_processor.sh which will
#    raise or cancel alerts as appropriate.
start_tail_task() {
   # ------------------------------------------------------------------------
   # We expect to be in the samples directory !, all files are relevant to
   # the samples directory.
   # ------------------------------------------------------------------------
   # Check that all the files we require exist.
   require_bin() {
      if [ ! -x $1 ];
      then
          echo "${myname}: Required file is missing: $1"
          exit 1
      fi
   }
   require_dat() {
      if [ ! -f $1 ];
      then
          echo "${myname}: Required file is missing: $1"
          exit 1
      fi
   }
   # All files relative to our script directory, which we have
   # changed directory into.
   require_bin "../log_parser"
   require_bin "log_parser_processor.sh"
   require_bin "../scripts/raise_alert.sh"
   require_bin "../scripts/end_alert.sh"
   require_dat "log_parser.sample_config"

   #------------------------------------------------------------------------
   # If we get here we can run, so do so.
   # ------------------------------------------------------------------------
   # Important note ! - many systems roll the /var/log/messages file daily
   # so you should after the cron job that does so run a job to stop the
   # current tail job and start a new one. This script can be used for that.
   # Note: we record the pid of our task ($$) rather than the pid of the processes we
   # spawn here as $! returns the last process started, which is the
   # log_parser_processor unfortunately, it's the tail we have to stop.
   # LINUX USERS: The GNU tail task can check if a log has been rolled, so if
   # the server is a Linux server you do not need to worry about the above
   # issue with log rolling.
   systype=`uname -s`
   if [ "${systype}." = "Linux." ];
   then
      tail --follow=name --retry --sleep-interval=5 --max-unchanged-stats=24 /var/log/messages 2>&1 \
	       | ../log_parser log_parser.sample_config | ./log_parser_processor.sh &
   else
      tail -f /var/log/messages | ../log_parser log_parser.sample_config | ./log_parser_processor.sh &
   fi
   #
   # Record PID details.
   # Note: on my linux box the PPID was set to 1 when the tasks backgrounded so
   # was no use to me in the stop script. I now include the user as well.
   iam=`whoami`
   echo "$$ ${iam}" > ${RUNPID_DIR}/monitor_var_log_parent.pid
} # end start_tail_task


#  Stops the tasks
#  Does so by stopping any tail of the file that was started
#  by the parent PID we have recorded.
#
# PURPOSE: On systems that roll /var/log/messages daily the
# tailing task will need to be stopped and restarted as the
# log files change. This script assists in that by stopping
# the previous task.
stop_tail_task() { 
   # ------------------------------------------------------------------------
   # We need the pid file to be able to work, so is it there.
   # ------------------------------------------------------------------------
   if [ ! -f ${RUNPID_DIR}/monitor_var_log_parent.pid ];
   then
      echo "${myname}: ${RUNPID_DIR}/monitor_var_log_parent.pid does not exist, no PID to stop !"
      exit 1
   fi

   # ------------------------------------------------------------------------
   # If we get here we can run, so do so.
   # ------------------------------------------------------------------------
   parentpid=`cat ${RUNPID_DIR}/monitor_var_log_parent.pid`

   # So much for plan A. When they get backgrounded on my version of linux the
   # parent pid switches to 1.
   # Do it by user. Have added the userid to the pid file also
   parentuser=`echo ${parentpid} | awk {'print $2'}`
   parentpid=`echo ${parentpid} | awk {'print $1'}`  # back to pid only as per original useage.

   # Problem 2: Changing the variable of killed below, within the loop DOES NOT ALTER
   # the variable outside the loop. I'm leavingthe logic in just in case you have a
   # useable version of bash, but I am implementing a file touch regime instead.
   # Note: original stuff is fine under ksh on solaris
   killed=0

   # ps -eo "ppid,pid,user,comm,args" | grep "tail -f \/var\/log\/messages" | grep "${parentpid}" | while read dataline
   systype=`uname -s`
   if [ "${systype}." = "Linux." ];
   then
      # This is BUGGY. If you have more than one under the same userid you could get the wrong one
      grepargs="tail --follow=name --retry --sleep-int"
   else
      grepargs="tail -f \/var\/log\/messages"
   fi
   ps -eo "ppid,pid,user,comm,args" | grep "${grepargs}" | grep "${parentuser}" | while read dataline
   do
      ppid=`echo ${dataline} | awk {'print $1'}`
      pid=`echo ${dataline} | awk {'print $2'}`
      ppuser=`echo ${dataline} | awk {'print $3'}`
      # Always check, if we had a small ppid it may have returned entries with it embedded,
      # we ensure it is exact.
      # if [ "${ppid}." = "${parentpid}." ];
      if [ "${ppuser}." = "${parentuser}." ];
      then
         # Stopping the tail task will close the pipes and stop the others
         kill ${pid}
         killed=1
         touch /tmp/${myname}_stop
      fi
   done
   # The old stuff
   # if [ "${killed}." = "1." ];  
   # then
   #    # Stopped something, ok to remove pid file
   #    rm -f ${RUNPID_DIR}/monitor_var_log_parent.pid
   # else
   #    echo "Unable to stop the /var/log tailing task, stop manually"
   # fi
   if [ -f /tmp/${myname}_stop ];  
   then
      # Stopped something, ok to remove pid file
      rm -f ${RUNPID_DIR}/monitor_var_log_parent.pid
      # and remove the touch file I have been forced to implement
      rm -f /tmp/${myname}_stop
   else
      echo "Unable to stop the /var/log tailing task, stop manually"
   fi
} # end stop_tail_task


#  Display the status of any log tasks
#  Finds the id of the tail task we started.
#
status_tail_task() { 
   # ------------------------------------------------------------------------
   # We need the pid file to be able to work, so is it there.
   # ------------------------------------------------------------------------
   if [ ! -f ${RUNPID_DIR}/monitor_var_log_parent.pid ];
   then
      echo "There is no PID file in the run directory. The log tailing"
      echo "task is probably not running."
      exit 1
   fi
   parentpid=`cat ${RUNPID_DIR}/monitor_var_log_parent.pid`
   parentuser=`echo ${parentpid} | awk {'print $2'}`
   parentpid=`echo ${parentpid} | awk {'print $1'}`  # back to pid only as per original useage.
   ps -eo "ppid,pid,user,comm,args" | grep "tail -f \/var\/log\/messages" | grep "${parentuser}" | while read dataline
   do
      ppid=`echo ${dataline} | awk {'print $1'}`
      pid=`echo ${dataline} | awk {'print $2'}`
      ppuser=`echo ${dataline} | awk {'print $3'}`
      # Always check, if we had a small ppid it may have returned entries with it embedded,
      # we ensure it is exact.
      # if [ "${ppid}." = "${parentpid}." ];
      if [ "${ppuser}." = "${parentuser}." ];
      then
         echo "Log tail task is process ${pid}"
         ps -ef | grep " ${pid} "
      fi
   done
} # end status_tail_task

# ==========================================================
#                      Mainline
# ==========================================================
check_control_path
case "$1" in
   start) 
          start_tail_task
          ;;
   stop)
          stop_tail_task
          ;;
   status)
          status_tail_task
          ;;
   *)
          echo "${myname} : start | stop | status"
          ;;
esac
exit 0
