#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------

# ****************************************************************
#
# process_busy_check.sh
#
# Check CPU activity, raise an alert if any thresholds are
# exceeded.
#
# We work off a default threshold, that if triggered will allow
# a check for an exception for the default. This allows us to
# have a low default threshold for the majority of jobs, but still
# allow alerts to be suppressed for those tasks that we expect a
# higher cpu useage from on a regular basis.
#
# Caution: designed for my job scheduler that handles stdout.
#          if running from cron customise to remove the informational
#          echo statements.
# ****************************************************************

ALERT_START_PROG="../scripts/raise_alert.sh"
ALERT_END_PROG="../scripts/end_alert.sh"

# ----------------------------------------------------------------
#                   Set the defaults here
# ----------------------------------------------------------------
threshold_default=2    # raise alert by default is above this %busy
threshold_current=${threshold_default}
keynum=0
myname=`basename $0`
mypath=`dirname $0`
alertcache="${mypath}/${myname}.alert_store"

# ----------------------------------------------------------------
# exception_table:
#
# There are processes that are allowed to be above the default 
# busy level, this is where we check for those and adjust the
# threshold if they are recognised.
# These are customised for known OK busy processes on my server,
# adjust for your environment.
# Note: ps truncates the command after 14 bytes so only check for 
# the first 14 bytes of a program or script name.
# ----------------------------------------------------------------
exception_table() {
  checkid=$1

  # kapm-idled - can be up at 98-100%
  # This is the idle process that sends HLT instructions to
  # the CPU when the server is idle. This is used to help
  # keep the CPU cool (believe it or not).
  if [ "${checkid}." = "kapm-idled." ];
  then
     threshold_current=100
  fi

  # r015_system_check.sh - needs 3% normally
  if [ "${checkid}." = "r015_system_che." ];
  then
     threshold_current=4
  fi

  # r015_process_busy_check.sh - needs 3% normally
  if [ "${checkid}." = "r005_process_bu." ];
  then
     threshold_current=4
  fi

  # The tripwire check is cpu intensive, allow 50%
  if [ "${checkid}." = "tripwire." ];
  then
     threshold_current=50
  fi

  # End of checks for adjusted thresholds.
} # end exception_table macro

# ----------------------------------------------------------------
#                      M A I N L I N E
#
#     List all running processes, and check the CPU busy
#
#  If the default threshold is exceeded check for an exception
#  being allowed for the program before raising an alert.
#
# Response from ps sample is...
# %CPU   PID COMMAND
#  0.0     1 init
#  0.0     3 keventd
# 45.9     4 kapm-idled
#  0.0     5 ksoftirqd_CPU0
# Note: command name is truncated after 14 bytes
#
# ----------------------------------------------------------------

# -----------------------------------------------------------------------------
# first clear any previous alerts
# -----------------------------------------------------------------------------
if [ -f ${alertcache} ];
then
   if [ -x ${ALERT_END_PROG} ];
   then
      alertlist=""
      cat ${alertcache} | while read keyname;
      do
         alertlist="${alertlist}${keyname}, "
         ${ALERT_END_PROG} ${keyname} "Alert cleared prior to next check"
      done
      echo "Cleared alerts for ${alertlist} prior to the new check"
   fi
   /bin/rm -f ${alertcache}
fi

# -----------------------------------------------------------------------------
# then check for any new process busy alerts needed
# -----------------------------------------------------------------------------
ps -e -o "%cpu,pid,comm" | grep -v "%CPU" | while read dataline
do
  threshold_current=${threshold_default}  # reset to default for each process check
  cpubusy=`echo "${dataline}" | awk '{print $1}'`
  # -- Only work with the whole number so we can do compares.
  cpubusy=`echo "${cpubusy}" | cut -d. -f1`
  # -- See if we are above the default threshold for an alert
  if [ ${cpubusy} -gt ${threshold_current} ];
  then
     program=`echo "${dataline}" | awk '{print $3}'`
     # -- If we are above see if this program is an exception to the default
     exception_table "${program}"
     # -- check again in case the threshold has changed
     if [ ${cpubusy} -gt ${threshold_current} ];
     then
        # -- Still above, an alert is needed.
        if [ -x ${ALERT_START_PROG} ];
        then
           keynum=$((${keynum}+1))
           key="CPU_PCT_${keynum}"
           text="${program} using ${cpubusy}% cpu, threshold is ${threshold_current}%"
           ${ALERT_START_PROG} ${key} "${text}" "warning"
           echo "Raising alert ${key}: ${text}"
           echo "${key}" >> ${alertcache}  # save to clear later
        else
          logger "*WARNING* ${text}"
          echo "*WARNING* ${text}"
        fi
     fi
  fi
done
exit 0
