#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------

# ---------------------------------------------------
#
# system_check.sh
#
# This is a merge of filesystem_check.sh and
# process_check.sh. I use this myself as it
# does it all.
# The process_check.sh and filesystem_check.sh
# are provided as they are easier for you to
# review to see how things work.
#
#
# Currently checks processes we expect to be
# running are running, and checks disk space
# used thresholds.
#
# For each check type any previous alerts are
# deleted, so we start fresh each time. This
# ensures that if the condition no longer exists
# the alerts are deleted from the centralised alert
# collection server without operations having to do
# so.
#
# Configuration files used are...
#   process_check.cfg - a program name on each line
#   filesystem_check.cfg - a filesysten name and max %full number per line
#
# Should be run at 15 minute intervals by the job
# scheduler, or by cron. You may want to do the
# filesystem checks only hourly if you have a relatively
# stable system.
#
# Cron may actually be safer as scheduler repeat jobs
# will delay if waiting on a newday action; I use
# the job scheduler though, with a seperate check in
# cron to make sure the job scheduler is running.
#
# Will...
#   clear any alerts it previously raised
#   raise new alerts as appropriate
# ---------------------------------------------------
#
# SET UP KEY VALUES, AND ENSURE THE SCRIPTS WE ARE
# EXPECTED TO RUN EXIST BEFORE DOING ANYTHING.
#
ALERT_START_PROG="../scripts/raise_alert.sh"
ALERT_END_PROG="../scripts/end_alert.sh"
# Get the location we are (and our files are).
myname=$0
mypath=`dirname ${myname}`
errorlog="${mypath}/system_check.errors" # a sink

# Check our binaries actually exist
if [ ! -f ${ALERT_START_PROG} ];
then
   logger "$0: NOT FOUND->${ALERT_START_PROG}"
   exit 1
fi
if [ ! -f ${ALERT_END_PROG} ];
then
   logger "$0: NOT FOUND->${ALERT_END_PROG}"
   exit 1
fi

# ---------------------------------------------------
# subproc - clear_alerts:
# for each alert recorded in the filename passed to
# this, clear the alert. Delete the alert file when
# we have finished.
# ---------------------------------------------------
clear_alerts() {
   # Clear any existing alerts, we will replace them if 
   # needed with new ones.
   if [ -f $1 ];
   then
      cat $1 | while read keyname;
      do
         $ALERT_END_PROG ${keyname} "Alert cleared prior to next check" 2>&1 >> ${errorlog}
      done
      /bin/rm -f $1
   fi
} # clear_alerts

# ---------------------------------------------------
# subproc - filehandle_check:
# Check that we still have a lot of free filehandles.
# ---------------------------------------------------
filehandle_check() {
   # Store in process check alert history
   alertcache="${mypath}/process_check.alert_store"

   # any existing alerts will have been cleared before the process_check called us

   # Only do anything if we have the expected linux file
   if [ -f /proc/sys/fs/file-nr ];
   then
      testvar=`cat /proc/sys/fs/file-nr`
      # fields are allocated, used and maximum permitted
      usedfh=`echo "${testvar}" | awk {'print $2'}`
      maxfh=`echo "${testvar}" | awk {'print $3'}`
      fhleft=$((${maxfh}-${usedfh}))
      # Value I have chosen is need at least 500 free
      if [ ${fhleft} -lt 500 ];
      then
         msgtext="Free filehandles low, ${fhleft} free, threshold 500"
         $ALERT_START_PROG PS_FILEHANDLE "${msgtext}" 2>&1 >> ${errorlog}
         echo "${keyname}" >> ${alertcache}  # save to clear later
      fi
   fi
} # filehandle_check

# ---------------------------------------------------
# subproc - process_check:
# Check processes that we expect to be running are
# in fact running.
# ---------------------------------------------------
process_check() {
   # Process check specific values
   processlist="${mypath}/process_check.cfg"
   alertcache="${mypath}/process_check.alert_store"

   # clear any existing alerts
   clear_alerts "${alertcache}"
   # Do the process checks, raise an alert for each process that
   # is not running.
   if [ -f ${processlist} ];
   then
      cat ${processlist} | while read cmdname;
      do
         testvar=`ps -eo args | grep "${cmdname}" | grep -v "grep"`
	     if [ "${testvar}." = "." ];  # not found by ps
         then
            keyname="PS_${cmdname}"
            $ALERT_START_PROG ${keyname} "Process ${cmdname} is not running" 2>&1 >> ${errorlog}
            echo "${keyname}" >> ${alertcache}  # save to clear later
         fi
      done
   else
      $ALERT_START_PROG PS_CFG_ERR "Configuration file ${processlist} does not exist" 2>&1 >> ${errorlog}
      echo "PS_CFG_ERR" >> ${alertcache}  # save to clear later
   fi

   # MID: 2003/06/05 Added a check for free file handles on linux also
   filehandle_check
} # process_check

# ---------------------------------------------------
# subproc filesystem_check:
# ---------------------------------------------------
filesystem_check() {
   # Process check specific values
   fslist="${mypath}/filesystem_check.cfg"
   alertcache="${mypath}/filesystem_check.alert_store"

   # clear any existing alerts
   clear_alerts "${alertcache}"
   if [ -f ${fslist} ];
   then
      cat ${fslist} | grep -v "#" | while read fs fslimit;
      do
         linecount=0
         pctfull=0
         df -k ${fs} 2>&1 | while read oneline;
         do
            if [ "${linecount}." = "0." ];
            then
               testval=`echo ${oneline} | awk {'print $1'}`
               if [ "${testval}." = "df:." ];
               then
                  keyname=`echo "${fs}" | tr "/" "_"`
                  keyname="FS${keyname}"
                  $ALERT_START_PROG ${keyname} "Filesystem ${fs} is not mounted" 2>&1 >> ${errorlog}
                  echo "${keyname}" >> ${alertcache}  # save to clear later
               fi
            fi
            if [ "${linecount}." = "1." ];
            then
               pctfull=`echo ${oneline} | awk {'print $5'} | awk -F% {'print $1'}`
               if [ ${pctfull} -gt ${fslimit} ];
               then
                  keyname=`echo "${fs}" | tr "/" "_"`
                  keyname="FS${keyname}"
                  $ALERT_START_PROG ${keyname} "Filesystem ${fs} is over ${fslimit}% full, at ${pctfull}%" 2>&1 >> ${errorlog}
                  echo "${keyname}" >> ${alertcache}  # save to clear later
               fi
            fi
            linecount=$((${linecount}+1))
         done
      done
   else
      $ALERT_START_PROG FS_CFG_ERR "Configuration file ${fslist} does not exist" 2>&1 >> ${errorlog}
      echo "PS_CFG_ERR" >> ${alertcache}  # save to clear later
   fi
} # filesystem_check

# ---------------------------------------------------
# Mainline
#
# See what checks have been requested, and run them.
# ---------------------------------------------------

# See what we are going to be checking
validcheck="NO"
cmd=$1
if [ "${cmd}." = "process." ];
then
   process_check
   validcheck="YES"
fi
if [ "${cmd}." = "filesystem." ];
then
   filesystem_check
   validcheck="YES"
fi
if [ "${cmd}." = "all." ];
then
   process_check
   filesystem_check
   validcheck="YES"
fi

# See if we did any work
if [ "${validcheck}." == "NO." ];
then
   $ALERT_START_PROG SYSCHK_ERR "Illegal command line option:${cmd}, no checks performed" 2>&1 >> ${errorlog}
   # Note: we don't record this alert in a file. I don't want the overhead os a
   # file just for this condition. If this condition exists it must be fixed.
fi

# Clean up the error log, we don't really want it, it was just to store all
# the newlines from the alert program.
/bin/rm -f ${errorlog}

exit 0
