#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#  The author, Mark Dickinson retains all rights to this software.
#
#  This version 1.0.0 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This version 1.0.0 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.A
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
# --------------------------------------------------------------------
#
# raise_alert.sh
#
# a helper script for batch jobs that need to raise alerts. It saves
# the user from having to know the configuration information for the
# alert server.
#
# Function: generate an alert message.
#
# Syntax: raise_alert.sh KEY "TEXT MESSAGE" [ critical | warning [ AUTOMATE <parms> ] ]
#
# Changes:
# MID: 2003/04/26 allowed for a third parameter to provide a 
#                 severity level other than the default of
#                 critical. Still defaults to critical if this
#                 new parameter is not provided so existing scripts
#                 will continue to work as expected. NOTE: I only
#                 changed this to allow warning events to be passed,
#                 so only 'critical' or 'warning' is allowed.
# MID: 2004/03/26 moved support for the automation toolkit inline
#                 so scripts don't need to do all the checking themselves,
#                 it can be all done here.
# --------------------------------------------------------------------

# We must NEVER write output to the screen as this script is
# intended to be run as a child task from a server process. The
# server process will not take kindly to anything trying to write
# to its stdout as the server will normally be a daemon with
# no terminal.
# Use this function insead of any echo commands.
write_message() {
   aa=`date`
   # Little fix. Some user defaults that createthe file when
   # it is written to were preventing other users from writing
   # to the file. So we'll pre-create it before the first
   # write for the day if we need to.
   if [ ! -f ${LOGPATH}/${LOG_FILE} ];
   then
      touch ${LOGPATH}/${LOG_FILE}
      if [ -f ${LOGPATH}/${LOG_FILE} ];
      then
         chmod 666 ${LOGPATH}/${LOG_FILE}
      fi
   fi
   # Then do the write that used to create as required.
   echo "${aa}: $*" >> ${LOGPATH}/${LOG_FILE}
}

# Load the configuration information
mypath=`dirname $0`
if [ -f ${mypath}/../etc/config.txt ];
then
   . ${mypath}/../etc/config.txt
else
  # We don't have any variables for write_message, so use logger here to
  # write to the syslogd task.
  logger "*FATAL* $0: No configuration files are available in ${mypath}/../etc !"
  exit 1
fi

# Get the required parameters
key=$1
text=$2
sevlevel=$3        # MID: 2003/04/26, added sevlevel parameter
if [ "${key}." = "." ];
then
   write_message "Syntax: raise_alert.sh KEY \"TEXT MESSAGE\"" 
   exit 1
fi
if [ "${text}." = "." ];
then
   write_message "Syntax: raise_alert.sh KEY \"TEXT MESSAGE\"" 
   exit 1
fi

# MID: 2004/03/26 - allow for optional automation request check
#                   if AUTOMATE is used there will be parms to pass to
#                   automation checks
autorecovery="$4"
if [ "${autorecovery}." = "AUTOMATE." ];
then
   # shift four fields already read, the parms are the rest.
   shift
   shift
   shift
   shift
   autoparms="$*"
fi

# MID: 2003/04/26 - new sevlevel checks
if [ "${sevlevel}." = "." ];
then
   sevlevel="critical"
fi

case "${sevlevel}" in
   "critical") ;;
   "warning")  ;;
   *)  write_message "Syntax: raise_alert.sh KEY \"TEXT MESSAGE\" [ critical | warning [ AUTOMATE <parms> ] ]" 
       exit 1
       ;;
esac
# If user passed, check it is OK

# Check the program exists
if [ ! -x ${INSTALLED_DIR}/${ALERTPROG} ];
then
   write_message "*FATAL* Program ${ALERTPROG} not found in ${INSTALLED_DIR}" 
   exit 1
fi

# MID: Added 2004/03/26 - automation checks inserted here so the calling
#                         script doesn't have to code them all.
if [ "${autorecovery}." = "AUTOMATE." ];
then
   if [ ! -x ${AUTOMATION_DIR}/${AUTOMATION_PROG} ];
   then
      write_message "*WARNING* Automation toolkit not installed, correctly.  Automation disabled"
      autorecovery=""
      # fall through to raise an alert as normal
   else
      ${AUTOMATION_DIR}/${AUTOMATION_PROG} ${autoparms}
      resultcode=$?
      if [ "${resultcode}." = "0." ]
      then
         exit 0        # we have finished
      fi
      # else fall through to raise an alert as normal
   fi
fi

# Execute the program, with a little fiddle to capture the output as if
# we pipe it directly to the logfile we keep getting blank lines when
# there is no error message to be written, so only write error messages
# to the file (if return-code <> 0).
bb=`${INSTALLED_DIR}/${ALERTPROG} ${PORT} ${SERVER_IPADDR} ${sevlevel} ${key} "${text}"`
result=$?
if [ "${result}." != "0." ];
then
   write_message "${bb}"
   write_message "*ERR: Unable to forward alert ${key}, ${text}"
else
   write_message "Raised : ${key} ${text}"
fi

# Always exit 0 to avoid problems with server tasks running this script
# as a child process.
exit 0 
