// package console.client;
/**
 *  Alert Console Application, (c) Mark Dickinson 2003
 *
 *  Part of the Alert Management Toolkit released by
 *  Mark Dickinson for the Linux environment.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.Vector;
import java.io.*;

import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
// import javax.util.Timer;
import java.util.Timer;
import java.util.TimerTask;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


import console.client.Constants;
import console.client.Alert_Console_MenuBar;
import console.client.SocketObject;
import console.client.ListItem;
import console.client.Alert_Console_List_MouseAdapter;

/*
 ---------------------------------------------------------------
 MyCellRenderer:
 Used by the JList to customise the display of our list into
 nice pretty colours.
 ---------------------------------------------------------------
*/
class MyCellRenderer extends JLabel implements ListCellRenderer {
   /** 
   Used to provide the ability to display the list items in
   different colours, and to place a border around the currently
   selected list item.
   */
   public MyCellRenderer () {
       // Don't paint behind the component 
       setOpaque(true);
   } 

   // Set the attributes of the
   //class and return a reference 
   public Component 
    getListCellRendererComponent(JList list, 
                                 Object value, // value to display 
                                 int index,    // cell index
                                 boolean iss,  // is selected
                                 boolean chf)  // cell has focus?
    { 
       // Set the text and color
       //background for rendering 
       setText(((ListItem)value).getValue( ));
       setBackground((( ListItem)value).getColor());
       // Set a border if the list
       //item is selected 
       if (iss) { 
          setBorder( BorderFactory.createLineBorder( Color.blue, 2));
       } else {
          setBorder( BorderFactory.createLineBorder( list.getBackground(), 2)); 
       } 
       return this;
    }
} /* MyCellRenderer */


/*
----------------------------------------------------------------
Class: Alert_Console_Application   ( M A I N )
----------------------------------------------------------------
*/

public class Alert_Console_Application implements Constants {
  /**
   * The main application environment class. Creates the
   * alert viewing screen, sockets needed and the object
   * listener items we need. Most classes created here
   * will link back to this environment to make use of the
   * objects created here.
   */

  public static void main (String[] args) {
     /**
      * Define the screen layout items, start the connection
      * to the remote server, implement all the event handlers
      * and threads, then start a repeat timer and timer thread.
      */

     // Use windows look and feel
     //     WindowUtilities.setNativeLookAndFeel();

     // ********************************************************
     // Get and check the aguments we expect, we must have a
     // hostname in parameter one and a port number in the
     // second parameter.
     // ********************************************************
     if (args.length < 2) {
        System.out.println( "Insuffient arguments: need hostname and port" );
        System.exit(1);
     }
     String remoteHost = args[0];
     int remotePort = StringToInt(args[1]);
     if (remotePort == 0) {
        System.out.println( "Invalid port number: "+args[1] );
        System.exit(1);
     }


     // ********************************************************
     // Create the main frame, and set the layout for the frame.
     // Add the main panel to it so we have somewhere to add our
     // many components.
     // ********************************************************
     JFrame ApplicationFrame = new JFrame( "Alert Console (c) Mark Dickinson, 2003 - Version 0.01 beta" );
     Container container = ApplicationFrame.getContentPane();
     container.setLayout(new FlowLayout());
     JPanel AlertPanel = new JPanel();


     // ********************************************************
     // We use a JList to display the alerts, define it here and
     // assign a model to it. Use a list model that allows for
     // updates.
     // This part of the JList setup needs to be done before we
     // start the tcpip socket listner thread as the thread needs
     // the JList model so it knows what to update.
     // ********************************************************
     DefaultListModel model = new DefaultListModel();
     final JList alertList = new JList(model);
     alertList.setCellRenderer( new MyCellRenderer()); 


     // ********************************************************
     // We communicate with the remote server across a TCPIP
     // interface, we should establist this now.
     // It MUST be done early on a we pass the connection
     // socket to child classes.
     //
     // Create the TCPIP Sockets we need, and classes to handle them. Note that
     // a socket read thread is created as part of this also.
     // ********************************************************
     final SocketObject serverConnection = new SocketObject(
                   remoteHost,remotePort,(DefaultListModel)model
           );
     if (!(serverConnection.isConnected)) {
        System.err.println("Couldn't connect!, terminating application");
        System.exit(1);
     }


     // ********************************************************
     // We should have a menu bar for the frame, create after
     // we have sockets to use.
     //
     // Add the menubar to the frame now, this needs to be done
     // after the socket connection has been created as some of
     // the menu options need to know about the socket to work.
     // ********************************************************
     JMenuBar UsefullMenu = new Alert_Console_MenuBar(serverConnection);
     ApplicationFrame.setJMenuBar(UsefullMenu);


     // ********************************************************
     // Back to the JList
     // Set the font we want.
     // ********************************************************
     // The below two lines work, but I'm happy with the standard font for now
     //     Font displayFont = new Font("CourierNew", Font.BOLD, 10);
     //     alertList.setFont(displayFont);


     // ********************************************************
     // add a listener and popup menu for mouse events in the JList
     // This will trigger on a mouse right-click.
     // Need to pass the list as well to get the data from it.
     // ********************************************************
     alertList.addMouseListener(
          new Alert_Console_List_MouseAdapter( alertList, serverConnection )
     );

     // ********************************************************
     // add a list selection handler to the JList
     // ********************************************************
     alertList.addListSelectionListener( new ListSelectionListener() {
         /**
         Anonymous ListSelectionListener for our JList. This has
         been obsoleted as all functionality has moved to the mouse
         event handler. The code block has been retained in case I 
         need to add further functionality later.
         */
         public void actionPerformed(ActionEvent e) {
             // nothing we want to do here
         }
         public void valueChanged( ListSelectionEvent e ) {
             //    int i = alertList.getSelectedIndex(); If we need the line number
             if (e.getValueIsAdjusting() == false)
             {
                 if (alertList.getSelectedIndex() != -1)
                 {
                        // -- don't need to do anything anymore, all actions
                        // -- are now done in the popup handler.
                 }  // of != -1
              } // of value adjusting = false
          } // of public void valuechanged
      }); // of add list selection handler


     // ********************************************************
     // Put the alert list into a scroll panel and add that
     // to the JPanel
     // ********************************************************
     JScrollPane alertpane = new JScrollPane( alertList );
// move below two to where show is done when we know window size 
//                                          hrz-width  vert-hight
//     alertpane.setPreferredSize( new Dimension( 740, 520 ) );
     AlertPanel.add( alertpane, BorderLayout.CENTER );

     // The Panel will be the content pane
     ApplicationFrame.setContentPane( AlertPanel );


     // ********************************************************
     // We want automatic refreshes every 60 seconds, so start
     // a timer task to issue the alert retrieve commands for
     // us.
     // Should use javax.util.timer but I can't find any doco
     // on that so using the dangerous AWT timer java.util.Timer.
     // ********************************************************
     final TimerTask refresh_loop = new TimerTask() {
        /** 
        Anonymous TimerTask thread class for handling our timer
        events. Sends a data refresh request to the server on
        each timer expiry to ensure our JList alert display has
        current events.
        */
        public void run() {
           // Send a refresh request
           serverConnection.writeBuffer( API_CMD_REFRESHLIST );
        }
     };
     Timer timer = new Timer();
     timer.scheduleAtFixedRate( refresh_loop, ONE_MINUTE, ONE_MINUTE ); // After, repeat delay


     // ********************************************************
     // Add a window listener for this application. We need to
     // be able to terminate as cleanly as possible.
     // ********************************************************
     ApplicationFrame.addWindowListener( new WindowAdapter() {
        /**
        Anonymous WindowAdapter class for the main application WindowListener.
        Just manages the windowClosing event to close the sockets, stop the
        timer thread and end the application.
        */
        // need a check on window size changes so scrollbar revalidate() can be called
        public void windowClosing(WindowEvent e) {
           if (serverConnection.isConnected) { serverConnection.disConnect(); }
           refresh_loop.cancel();  // stop timer task
           System.exit(0);
        }
     });


     // ********************************************************
     // Show the frame before we do any data collection
     // requests.
     // ********************************************************
     ApplicationFrame.pack();
     ApplicationFrame.setBounds(50,50,800,600);
     // Set the size of the internal list here, we need to find out
     // how to resize this when the main window is resized.
                                 // width - 60, hight - 80
     alertpane.setPreferredSize( new Dimension( 740, 520 ) );
     ApplicationFrame.setVisible(true);
     ApplicationFrame.show();

     // ********************************************************
     // Note: timer waits one minute so our threads can start up...
     // Do the initial display rather than wait for timer
     // ********************************************************
     serverConnection.writeBuffer( API_CMD_REFRESHLIST );
  }

  // Java just doesn't have a routine to convert a number held in a string
  // to an integer, we need one so create one here. It will only support
  // the ASCII character set. I can't afford an MVS mainframe to test any
  // EBCDIC handling and don't intend to bother coding it.
  static public int StringToInt( String numberInString ) {
     int totalVal = 0;
     char testByte = '0';
     if ((int)testByte == 48) { // ascii character table
        char[] zz = numberInString.toCharArray();
        totalVal = 0;  // just in case it's not initialised to zero
        for (int i = 0; i < numberInString.length(); i++)
        {
           totalVal = (totalVal * 10) + ((int)zz[i] - 48);
        }
     } else { // must be ebcdic character set, not implemented
        System.out.println( "EBCDIC Character Set is not supported");
        totalVal = 0;
     }
     return totalVal;
  } // StringToInt

}
