/*
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
*/

/*
# -----------------------------------------------------------
#
#  alert_collector_server.c
#
# The function of this task is to record events written to it
# from other applications. It provides an alert retrieval and
# delete interface for console applications.
#
# It allows alerts from multiple servers to be collected in 
# one central place for monitoring.
#
# It will maintain up to MAX_ASRV_MEM_TABLESIZE alerts in memory,
# when this is exceeded it will start recording alerts to
# an overflow disk file. Console applications cannot maniplulate
# or view alerts in the overflow file, these will become available
# as in memory alerts are deleted.
#
# It uses default port of 9003 to listen on, this can be overriden
# at startup. It also be default listens on all interfaces, this
# may also be overwritten at startup to make it listen to a single
# internal interface if other interfaces connect to the public
# network.
#
# Syntax:
#   Linux:   alert_collector_server [<port> [<interface-ipaddr>]]
#   Solaris: alert_collector_server [<port> [<interface-ipaddr>]] &
# The & is required on Solaris to background it as the Linux daemon
# call won't compile on solaris.
# If you specify an interface-ipaddr to listen on you must also
# provide the port number.
#
# Additional notes:
# - uses lazy tcpip calls, the buffer is up to the NL. as such we
#   translate any imbedded NL (\n) chanracters to ^ before sending.
#   Any recieving console application must handle this.
#   We DO NOT EXPECT any NL characters in input, as we will stop
#   reading from the socket at a NL.
#   THIS IS IMPORTANT as all my apps expect a header block and there
#   were probs writing to the java console where the first line was
#   processed ok in a listall but all the rest read as individual
#   lines without the header.
#
# Changes:
# MID - 2004/04/20 - Added the fromOverflow and noBuffer flags to manage
#                    processing from the overflow files. When an ack or delete
#                    was being read from the overflow file and no matching entry
#                    found it was being put back on the overflow queue. Now we
#                    track if an event was from the overflow file and do not put
#                    the event back into the file if it was read from there.
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <time.h>
#include <sys/utsname.h>
#include <limits.h>
#include <signal.h>

/* import the file that identifies if we are LINUX or SOLARIS */
#include "system_type.h"

/* These are in different places depending on server type */
#ifdef SOLARIS
#define __STDC__
/* #include <sys/varargs.h> */
#include <stdarg.h>
/* #undef __STDC__ */
#else
#include <stdarg.h>
#endif

/* Exit codes from 'die' */
#define DIE_INTERNAL    1
#define DIE_TCPIP_SETUP 2

#ifndef SHUT_RDWR
#define SHUT_RDWR 3
#endif

#define MAX_ASRV_STACK   32
#define MAX_ASRV_CLIENTS 64
#define MAX_ASRV_MEM_TABLESIZE 50

#include "../common/constants.h"

/*
typedef struct {
   char buffer[4096];
} mpz_t;
*/
typedef unsigned char MPZ_CHAR;
typedef MPZ_CHAR MPZ_BUF[4096];
typedef MPZ_BUF mpz_t;
/* typedef char[4096] mpz_t; */

/* static mpz_t *stack[MAX_ASRV_STACK]; */
mpz_t * mpz_t_stack[MAX_ASRV_STACK];
char * stack;
static int sp = 0;

typedef struct {
   FILE  *rx;
   FILE  *tx;
   mpz_t **stack;     /* stack array   */
   int   sp;          /* stack pointer */
   char  remoteaddr[50];
   int   remoteport;
   char accept_unsol;    /* write unsolicted messages to this connection Y/N */
   char connect_service; /* F=alert forwrder, C=console app */
} ClientInfo;

ClientInfo client[MAX_ASRV_CLIENTS+1];

#include "../common/constants.h"
#include "../common/shared_api_header.h"
#include "../common/alertserver_api_def.h"
#include "../common/console_api_def.h"


/* ------------------------------------------------------
   Used for message logging
   ------------------------------------------------------ */
FILE *msg_log_handle = NULL;
char msg_logfile_name[17] = "log_YYYYMMDD.log\0";
int logfile_roll_time_last_day = 0;

/* ------------------------------------------------------
   Used for message archiving when the msg queue is full.
   ------------------------------------------------------ */
FILE *event_log_handle = NULL;
char event_logfile_name[15] = "evt_buffer.dat\0";
long next_evt_record_to_read = 0;
int unlink_permitted = 0;

/* ------------------------------------------------------
   Used for memory buffering of messages.
   ------------------------------------------------------ */
message_data_def memory_table[MAX_ASRV_MEM_TABLESIZE];
long last_table_entry = 0;

/* *********************************************************
   Utility Routines.
   ********************************************************* */

/* A couple of forward declarations we need */
void SERVER_roll_log_files();
void process_alert_message( message_data_def *buf, int fromOverflow );

#include "../common/shared_utils.c"

/* Done with debugging
#define DEBUG_ON
#define DEBUG_HEADERS
*/


/* =========================================================
 * myprintf:
 * 
 * Used to write log messages. Accepts parametrs in same
 * format as printf.
   ========================================================= */
void myprintf( const char *fmt, ... ) {
        int n;
        char *p, *p2;
        va_list ap;
        time_t time_now;
        char time_now_text[50];
        struct tm time_var;
        struct tm *time_var_ptr;

        /* Used for multiple things, set right at start */
        time_now = time(0);   /* also used in logging text */

        /* For each message now check to see if we need to write 
           to a new days log file, this has been moved here so it
           is only checked on each message write (used to be done
           every 1.3 seconds in the mainline) */
        time_var_ptr = &time_var;
        time_var_ptr = localtime(&time_now);
        if (time_var_ptr->tm_mday != logfile_roll_time_last_day) {
           SERVER_roll_log_files();
        }

        /* Then do the main body of the procedure */
        if ( (p = malloc(512)) == 0 ) { /* don't expect, or allow msgs > 511 bytes */
                fprintf( stdout, "Unable to allocate 512 bytes of memory for message display\n" );
                return;  /* can't even log an error */
        }

        /* The va_* routines are used when passing an unknown number of
         * variables, and also the variables are of unknown types
         * normally used as
         *    void va_start(va_list,last-parm-before-list)
         *    type va_arg(va_list,ap,type)
         *    void va_end(va_list)
         * we can replace the va_arg hard work with vsnprintf as we know
         * we are being called with printf parameters
         */
         va_start(ap,fmt);
         n = vsnprintf( p, 511, fmt, ap );   /* allowing only 511 bytes so we can have a null in 512 */
         va_end(ap);

         /* n will be -1 on errors from vsnprintf, or may be > buffsize to be
          * used if it is not going to fit in the buffer */
         if ((n > -1) && (n < 511)) {
            if (msg_log_handle != NULL) {  /* Log file is open */
               snprintf( time_now_text, 49, "%s", ctime(&time_now) );
               p2 = (char *)&time_now_text;
               p2 = p2 + (strlen(time_now_text) - 1);
               *p2 = '\0';
               fprintf( msg_log_handle, "%s %s", time_now_text, p );
               /* Flush to ensure its written to the log file immediately
                * as I've noticed that with caching on my machine with a low
                * number of messages it can take hours for the log to be
                * updated otherwise */
               fflush( msg_log_handle );
               clearerr( msg_log_handle );
            }
            else {  /* no log file open, use stdout */
               fprintf( stdout, "%s", p );
            }
         }
         else {
            myprintf( "*ERR: Message of > 512 bytes passed, discarded\n" );
         }
         free(p);
         return;
}  /* myprintf */


#ifdef DEBUG_ON
#include "debuglib.c"
#endif
#ifdef DEBUG_HEADERS
#include "debuglib.c"
#endif


/* =========================================================
 * message_from_forwarding_application:
 *
 * More than one check is done on whether a message is from
 * a forwarding server, so put them all here so we can keep 
 * a list of what the known ones are in one place.
 * Return 1 if a forwarding application, 0 if not.
   ========================================================= */
int message_from_forwarding_application( common_header_def *header_block ) {
  if ( ( memcmp(header_block->productid, "JOBSCHED", 8) == 0 ) || /* job scheduler */
       ( memcmp(header_block->productid, "GENALERT", 8) == 0 ) || /* command line alert generator */
       ( memcmp(header_block->productid, "RELOADER", 8) == 0 ) )  /* alert reloader task */
  {
     return 1;
  }
  else {
     return 0;
  }
} /* message_from_forwarding_application */


/* =========================================================
 * message_from_console_application:
 *
 * See if the reqquest is from a console application.
 * If not return 0.
 * If it is return the number of lines the console will
 * support.
   ========================================================= */
int message_from_console_application( common_header_def *header_block,
                                      int *consolelinelen ) {
   char *ptr;
   int numlines, numbytesinline;

#ifdef DEBUG_ON
  myprintf("DEBUG: Console check, productid=%s\n", header_block->productid );
#endif
  if ( memcmp(header_block->productid, "CONSOLE", 7) == 0 ) {  /* generic */
      /* see if a line number was provided ,ie: CONSOLE24, CONSOLE48 etc */
      /* Changed 20Mar2003 to allow linewidth also ie:CONSOLE24_80 */
      ptr = (char *)&header_block->productid + 7;
      if ((*ptr != ' ') && (*ptr != '\0') && (*ptr != '\n')) {
         numlines = 0;
         numlines = atoi(ptr);
         if (numlines == 0) {
            numlines = 23; /* default to generic 24 line */
            myprintf( "WARN: Illegal # console lines in productid from %s, defaulting to 24 lines\n",
                      header_block->hostname );
         }
         else if (numlines > 2) {
            numlines = numlines - 1; /* leave room for cmd line */
         }
         /* see if optional linelen in value also */
         ptr = ptr + 2;  /* move over 2 byte numlines */
         if (*ptr == '_') {
            ptr++;
            numbytesinline = atoi(ptr);
            if ((numbytesinline == 0) ||
                (numbytesinline > ASRV_MAX_CONSOLE_LINE_LEN)) {
               numbytesinline = ASRV_MAX_CONSOLE_LINE_LEN;
               myprintf( "WARN: Illegal # console linelen in productid from %s, defaulting to %d lines\n",
                      header_block->hostname, ASRV_MAX_CONSOLE_LINE_LEN );
            }
            *consolelinelen = numbytesinline;
         }
         return( numlines );
      }
      else { /* generic 24 line console, return 23 */
          return( 23 );
      }
  }
  else {  /* not a CONSOLE keyword */
     return( 0 );
  }
} /* message_from_console_application */



/* =========================================================
 * SERVER_roll_log_files:
 * 
 * Roll the message log files to a new days date.
   ========================================================= */
void SERVER_roll_log_files() {
      time_t time_number;
      struct tm time_var;
      struct tm *time_var_ptr;
      char datenow[10];

      /* Work out the date for the logfile name */
      time_var_ptr = &time_var;
      time_number = time(0);
      time_var_ptr = localtime(&time_number);	
      time_var_ptr->tm_year = time_var_ptr->tm_year + 1900;  /* years returned as count from 1900 */
      time_var_ptr->tm_mon = time_var_ptr->tm_mon + 1;       /* months returned as 0-11 */
      snprintf( datenow, 9, "%.4d%.2d%.2d",
                time_var_ptr->tm_year, time_var_ptr->tm_mon, time_var_ptr->tm_mday );
      snprintf( msg_logfile_name, 17, "log_%s.log", datenow );

	  /* Set this NOW. Was going into an endless loop trying to write
	   * the close message becuase the close message detected a date 
	   * difference and kept calling us back again. */
	logfile_roll_time_last_day = time_var_ptr->tm_mday;

     /* Close any open logfile */
	if (msg_log_handle != NULL) {
         myprintf( "Closing log file, rolling over to a new days log\n" );
         fclose( msg_log_handle );
		 msg_log_handle = NULL;
      }

      /* Open the new logfile */
      if ( (msg_log_handle = fopen( msg_logfile_name, "a+" )) == NULL ) {
         myprintf( "*ERR: Unable to open log file %s, continuing using STDOUT for messages\n", msg_logfile_name );
         return;
      }

      /* Work out the next logfile roll time */
      time_number = time(0);
	time_number = time_number + (60 * 60 * 24);
      time_var_ptr = localtime(&time_number); 
      time_var_ptr->tm_hour = 0; 
      time_var_ptr->tm_min = 0; 
      time_var_ptr->tm_sec = 1; 
} /* SERVER_roll_log_files */


/* =========================================================
 * shutdowm_immed:
 * 
 * Perform a server shutdown. Called on a shutdown request.
   ========================================================= */
static void shutdown_immed( int rc ) {
   int z;

   if (msg_log_handle != NULL) {
      myprintf(  "----: Shutdown in progress\n" );
   }
   
   /* Free all memory stacks, Close all open sockets */
   for ( z = 0; z < MAX_ASRV_CLIENTS; ++z ) {
      stack = (char *) client[z].stack;
      if (stack != NULL) {
         sp = client[z].sp;
         while (sp > 0) {
            free( &stack[--sp] );
         }
         free( stack );
      }
      if (client[z].tx != NULL) {
#ifdef DEBUG_ON
         myprintf(  "INFO: Closing session to %s\n", client[z].remoteaddr );
#endif
         fclose( client[z].tx );
         shutdown(fileno(client[z].rx),SHUT_RDWR);
         fclose( client[z].rx );
      }
      client[z].stack = NULL;
      client[z].sp = 0;
      client[z].rx = NULL;
      client[z].tx = NULL;
   }

   /* close all the files we have open */
   if (event_log_handle != NULL) {
      fclose( event_log_handle );
   }
   if (msg_log_handle != NULL) {
      myprintf(  "----: ALL OK, Server shutdown has completed.\n" );
      fclose( msg_log_handle );
   }
   exit( rc );
} /* shutdown_immed */


/* =========================================================
 * die:
 * 
 * provide a fast-stop that does a bit of a cleanup. This is called if we
 * encounter a non-recoverable error and need to bail out of the program.
   ========================================================= */
static void die(const char *msgtext, int exitcode) {
   perror( msgtext );
   myprintf(  "*ERR: ***FATAL*** A fatal server error has occurred. Cannot continue.\n" );
   myprintf(  "*ERR: %s\n", msgtext );
   myprintf(  "INFO: Server has aborted.\n" );
   shutdown_immed( exitcode );
   exit( exitcode); /* should not get here, but catch in in case
                     * there wasn't room on stack for call */
} /* die */


/* =========================================================
 *  get_sock_addr: 
 * 
 *  From a passed socket file handle, obtain the socket address
 *  and port number.
 *  input:  buf - pointer to a buffer to hold the result
 *          bufsize - maximum size allowed for result
 *  output: buf - updated with result
 *  returns: buf if ok, NULL if an error
   ========================================================= */
char * get_sock_addr( int sockfh, char *buf, size_t bufsiz ) {
   int z;
   struct sockaddr_in adr_inet;    /* AF_INET */
   int len_inet;                   /* length */

   len_inet = sizeof adr_inet;
   z = getsockname( sockfh, (struct sockaddr *)&adr_inet, &len_inet );
   if (z == -1) return NULL;

   /* convert address into a string form for display */
   snprintf(buf,bufsiz,"%s:%u",inet_ntoa(adr_inet.sin_addr),
            (unsigned)ntohs(adr_inet.sin_port));
   return buf;
} /* get_sock_addr */


/* =========================================================
 *  From a passed socket file handle, obtain the socket address
 *  and port number of the remote (peer) end of the connection.
 *  That is... find out who is talking to us.
 *  input:  buf - pointer to a buffer to hold the result
 *          bufsize - maximum size allowed for result
 *  output: buf - updated with result
 *  returns: buf if ok, NULL if an error
   ========================================================= */
char * get_remote_addr( int sockfh, char * buf, size_t bufsize ) {
   int z;
   struct sockaddr_in adr_inet;    /* AF_INET */
   int len_inet;                   /* length */

   len_inet = sizeof adr_inet;
   z = getpeername( sockfh, (struct sockaddr *)&adr_inet, &len_inet );
   if (z == -1) return NULL;

   /* convert address into a string form for display */
   z = snprintf(buf,bufsize,"%s:%u",inet_ntoa(adr_inet.sin_addr),
                (unsigned)ntohs(adr_inet.sin_port));
   if ( z == -1 ) return NULL;     /* buffer to small */

   return buf;
} /* char * get_remote_addr */


/* *********************************************************
   Routines to process the alerts being forwarded from remote
   hosts to this server.
   ********************************************************* */


/* =========================================================
 * alert_record_event:
 * records each alert message recieved into the daily 
 * message log file.
   ========================================================= */
void alert_record_event( message_data_def *buffer ) {
   int  linelen;
   char msgbuffer[129]; /* 128 + null */

   linelen = snprintf( msgbuffer, 128,
             "AINF: %s %s %c%c %s %c %s %s",
             buffer->msg_header.hostname,
             buffer->msg_header.source_date_time,
             buffer->msg_header.messagetype[0],
             buffer->msg_header.messagetype[1],
             buffer->msg_header.unique_key,
             buffer->datapersists,
             buffer->datastr,
             buffer->optdata );
   msgbuffer[128] = '\0';  /* in case too much data for a null to be added */
   myprintf( "%s\n", msgbuffer );
   return;   
} /* alert_record_event */


/* =========================================================
 * alerts_recreate_overflow_file:
 * called from alert_delete if all the records from the
 * overflow file are now in the memory table. We take the
 * oportunity to delete/recreate the file as it may have
 * grown quite large.
   ========================================================= */
void alerts_recreate_overflow_file( void ) {
#ifdef DEBUG_ON
     myprintf( "DEBUG:alerts_recreate_overflow_file\n" );
#endif
   if (unlink_permitted == 0) {
#ifdef DEBUG_ON
        myprintf( "DEBUG:overflow file has no data, skipping\n" );
#endif
      return; /* we already were at 0 butes, don't waste disk IOs */
   }
#ifdef DEBUG_ON
     myprintf( "DEBUG:alerts_recreate_overflow_file, resetting file\n" );
#endif
   fclose( event_log_handle );
   if (unlink( event_logfile_name ) != 0) {
      myprintf( "*ERR: Unable to delete overflow file, aborting\n" );
	  shutdown_immed( 1 );
   }
   unlink_permitted = 0;
   next_evt_record_to_read = 0;
   if ( (event_log_handle = fopen( event_logfile_name, "a+" )) == NULL ) {
      myprintf( "*ERR: Unable to re-open alert overflow buffer file %s, aborting!\n", event_logfile_name );
      die( "IO error on alert overflow file", DIE_INTERNAL );
   }
   return;
} /* alerts_recreate_overflow_file */


/* =========================================================
 * alerts_num_buffered:
 * There are a lot of routines that need to know if there
 * are any records still in the overflow buffer file
 * waiting to be read. This routine returns the number
 * we are still waiting on.
   ========================================================= */
long alerts_num_buffered( void ) {
  long overflow_size, records_buffered;
  int  iostatus;

  /* first see if there are any messages in the buffer file,
   * we need this for a lot of checks here so do it first */
  iostatus = fseek( event_log_handle, 0, SEEK_END );
  if (iostatus == 0) {
     overflow_size = ftell( event_log_handle );
	 overflow_size = ( overflow_size / sizeof(message_data_def) );
     records_buffered = overflow_size - next_evt_record_to_read;
#ifdef DEBUG_ON
     myprintf( "DEBUG: Currently %d records in overflow file\n", records_buffered );
#endif
	 if (records_buffered < 0) { records_buffered = 0; } /* saftey net */
  }
  else {
     records_buffered = 0;
#ifdef DEBUG_ON
     myprintf( "DEBUG: IO error checking overflow file, will indicate 0 records buffered\n" );
#endif
  }
  return( records_buffered );
} /* alerts_num_buffered */


/* =========================================================
 * alert_write_to_bufferfile:
 * Called when the memory tables are full and there were
 * no info messages to be deleted. Store the record for
 * later retrieval at EOF.
   ========================================================= */
void alert_write_to_bufferfile( message_data_def *buffer ) {
   int io_status, byteswrite;

   io_status = fseek( event_log_handle, 0, SEEK_END );
   if (io_status == 0) {
      /* MUST set reply expected to no, alerts buffered into the
       * file will already have been replied to by the time they
       * are read back from the file again. */
      buffer->msg_header.expect_reply = 'N';
      /* save the alert to the overflow file */
      byteswrite = fwrite( buffer, sizeof( message_data_def ), 1, event_log_handle );
      if (byteswrite != 1) { /* 1 element written */
         myprintf( "WARN: Unable to store events in buffer file, disk full ?" );
      }
      unlink_permitted = 1; /* we have stuff in the file, so unlink is allowed  when it's cleared */
      /* note: just set value above, it's quicker that checking if 0 every
	   * iteration */
   }

   return;
} /* alert_write_to_bufferfile */


/* =========================================================
 * alerts_sort:
 * This should not be neccesary as currently we accept and
 * record events chronologically. May be required later.
   ========================================================= */
void alerts_sort( message_data_def *buffer ) {
  return;
} /* alerts_sort */


/* =========================================================
 * alerts_compress:
   ========================================================= */
int alerts_compress( void ) {
  int i, j, loop_trap, deleted;

  /* ensure we have entries */
  if (last_table_entry == 0) { return(0); } /* nothing to do */

  deleted = 0;
  loop_trap = 0;
  i = 0;
  while (i < last_table_entry) {
     if (memcmp(memory_table[i].msg_header.messagetype,"DD",2) == 0) {
        deleted++;
        j = i + 1;
        for (j = i; j < (last_table_entry - 1); j++ ) {
           memcpy( memory_table[j].msg_header.productid, 
                   memory_table[j+1].msg_header.productid,
                   sizeof( message_data_def ) );
        }
        last_table_entry--;
     }
     else {
       i++;
     }
     /* Play safe, does no harm */
     loop_trap++;
     if (loop_trap > MAX_ASRV_MEM_TABLESIZE) {
        myprintf( "*ERR: Looping in alerts_compress, programmer error. Aborting compress\n" );
        return;
     }
  }

  return( deleted );
} /* alerts_compress */


/* =========================================================
 * alert_lookup:
 * see if an alert exists in our memory table. We do NOT 
 * scan the disk overflow buffer, if we are overflowing the
 * message will be written to the file and not come here.
   ========================================================= */
int alert_lookup( common_header_def *buffer ) {
  int i;

  /* If no entries, nothing to check */
  if (last_table_entry == 0) {
     return -1;
  }

  /* Check the memory table */
  for (i = 0; i < last_table_entry; i++) {
     if (
       (memcmp(memory_table[i].msg_header.hostname,buffer->hostname,MAX_ASRV_HOSTNAME_LEN) == 0) &&
       (memcmp(memory_table[i].msg_header.unique_key,buffer->unique_key,MAX_ASRV_KEY_LEN) == 0)
        )
     {
        return i;  /* found the match */
     }
  }

  /* Not found */
  return -1;
} /* alert_lookup */


/* =========================================================
 * alert_add:
   ========================================================= */
void alert_add( message_data_def *buffer ) {
  int i, entry_deleted, memory_index;

  /* Ensure the fields sent to us are terminated as expected */
  buffer->msg_header.productid[ASRV_PRODUCT_ID_LEN] = '\0';
  buffer->msg_header.source_date_time[MAX_ASRV_DATE_LEN] = '\0';
  buffer->msg_header.hostname[MAX_ASRV_HOSTNAME_LEN] = '\0';
  buffer->msg_header.unique_key[MAX_ASRV_KEY_LEN] = '\0';

  /* This may be a repeat of an existing alert, check
   * that, if it is delete the original and and the
   * new one. This can only be done if 'in-memory'. */
   memory_index = alert_lookup( (common_header_def *)buffer );
   if (memory_index >= 0) { /* a match was found, replace existing alert */
#ifdef DEBUG_ON
      myprintf( "DEBUG: Matching alert found, replacing alert %d.\n", memory_index );
#endif
      memcpy( memory_table[memory_index].msg_header.productid,
              buffer->msg_header.productid, sizeof(message_data_def) );
      return;
   }

  /* If no table entries free see if we can cleanup table */
  if (last_table_entry >= MAX_ASRV_MEM_TABLESIZE) {
#ifdef DEBUG_ON
     myprintf( "DEBUG:alert_add, mem table full, attempting compress\n" );
#endif
     entry_deleted = 0;
     /* Delete the oldest (top of table) info message */
     for (i = 0; i < last_table_entry; i++) {
        if ((memcmp(memory_table[i].msg_header.messagetype,"IO",2) == 0) &&
            (entry_deleted == 0) ) 
        {
           memory_table[i].msg_header.messagetype[0] = 'D';
           memory_table[i].msg_header.messagetype[1] = 'D';
           entry_deleted = 1;
        }
     }
     /* If not possible need to add to overflow buffer file */
     if (entry_deleted > 0) {
#ifdef DEBUG_ON
     myprintf( "DEBUG:alert_add, mem table entry released, compressing\n" );
#endif
        alerts_compress();
     }
     else {
#ifdef DEBUG_ON
     myprintf( "DEBUG:alert_add, mem table cannot be compressed, writing to file\n" );
#endif
        alert_write_to_bufferfile( buffer );
        return;  /* Written to file, no more to do */
     }
  }

  /* Check that the compress worked! */
  if (last_table_entry >= MAX_ASRV_MEM_TABLESIZE) {
     myprintf( "*ERR: Full memory table and memory table compress failed\n" );
     myprintf( "*ERR: To avoid server failure server is flushing memory tables\n" );
     last_table_entry = 0;
     return;
  }

#ifdef DEBUG_ON
  myprintf( "DEBUG: Adding new alert at postion %d.\n", last_table_entry );
#endif

  /* We have a spare slot now so add it */
  memcpy( memory_table[last_table_entry].msg_header.productid,
          buffer->msg_header.productid, sizeof(message_data_def) );
  last_table_entry++;
#ifdef DEBUG_ON
  myprintf( "DEBUG: Incremented last_table_entry, now at postion %d.\n", last_table_entry );
#endif
  return;
} /* alert_add */


/* =========================================================
 * alert_delete_all_for_host:
 * Generally this should only be requested when a host is
 * shutting down, to clean up after itself.
   ========================================================= */
void alert_delete_all_for_host( message_data_def *buffer, int noBuffer ) {
  int sink_var, i;

  /* If no entries, nothing to do */
  if (last_table_entry == 0) {
     return;
  }

  /* If there are records in the buffer overflow file we
   * may not be able to automatically delete all the alerts
   * for the hostname, so in the case of buffered records we
   * will also write the request into the overflow file so it
   * will be processed again when the records in the overflow file
   * are read
   */
  if ( (alerts_num_buffered() > 0) && (noBuffer == 0)) {
     alert_write_to_bufferfile( buffer );
  }

  /* Check the memory table, delete matching host entries */
  for (i = 0; i < last_table_entry; i++) {
     if (memcmp(memory_table[i].msg_header.hostname,buffer->msg_header.hostname,MAX_ASRV_HOSTNAME_LEN) == 0)
     {
        strcpy(memory_table[i].msg_header.messagetype, "DD" ); /* schedule for delete */
     }
  }

  /* Compress the memory table now, and load any buffered entries into the
   * space free'd up */
  sink_var = alerts_compress();
} /* alert_delete_all_for_host */


/* =========================================================
 * alert_delete:
   ========================================================= */
int alert_delete( message_data_def *buffer, int noBuffer ) {
  long overflow_size, records_buffered;
  int memory_index;

  /* first see if there are any messages in the buffer file,
   * we need this for a lot of checks here so do it first */
  records_buffered = alerts_num_buffered();

  /* locate the position of the message in the memory table */
  memory_index = alert_lookup( (common_header_def *)buffer );
  
  /* if we can't find it it may be in the buffer file ? */
  /* If from a console user entred against a displayed memory record,
   * so another task must have deleted it */
  if ((memory_index < 0) && (records_buffered > 0) &&
      (message_from_forwarding_application((common_header_def *)&buffer->msg_header.productid) == 1) )
  {
     /* it could be, just write it to the buffer file */
     /* hmm, but could get an alert stuck in the buffer file ? */
     if (noBuffer == 0) {
        alert_write_to_bufferfile( buffer );
     }
     return( 0 ); /* nothing deleted and nothing more we can do */
  }
  else if (memory_index < 0) {
     myprintf( "WARN: message not found to delete, requested from %s\n",
               buffer->msg_header.hostname );
     return( 0 );  /* nothing to do */
  }

  /* If here, we found the memory entry */
  strcpy(memory_table[memory_index].msg_header.messagetype, "DD" );

  /* return the number of entries deleted */
  return( alerts_compress() );
} /* alert_delete */


/* =========================================================
 * alert_acknowledge:
 * For now just acknowledge the alert on the local server.
   ========================================================= */
void alert_acknowledge( message_data_def *buffer, int noBuffer ) {
  int memory_index;
  long records_buffered;

  records_buffered = alerts_num_buffered(); /* any unread in overflow file ? */

  /* locate the position of the message in the memory table */
  memory_index = alert_lookup( (common_header_def *)buffer );
  
  /* if we can't find it it may be in the buffer file ? */
  /* This is only acceptable for messages from forwarding servers
     as a console app will not have seen the entry displayed to
     be acknowledged */
  if ((memory_index < 0) && (records_buffered > 0) &&
      (message_from_forwarding_application((common_header_def *)&buffer->msg_header.productid) == 1) )
  {
     /* it could be, just write it to the buffer file */
     /* hmm, but could get an alert stuck in the buffer file ? */
     if (noBuffer == 0) {
        alert_write_to_bufferfile( buffer );
     }
     return; /* and nothing more we can do */
  }
  else if (memory_index < 0) {
     myprintf( "WARN: message not found to acknowledge, requested from '%s', key '%s'\n",
               buffer->msg_header.hostname, buffer->msg_header.unique_key );
     return;  /* nothing to do */
  }

  /* If here, we found the memory entry, set to acknowledged */
  strcpy(memory_table[memory_index].msg_header.messagetype, "AK" );
  return;
} /* alert_acknowledge */


/* =========================================================
 * alert_check_overflow_file:
 * Called when we wish to see if the buffer overflow file
 * is empty, if it is we want to resize it back to 0.
   ========================================================= */
void alert_check_overflow_file( void ) {
   /* see if any more in the overflow file,
    * if not resize it back to zero (delete) */
   if (alerts_num_buffered() == 0) {
      alerts_recreate_overflow_file();
   }
   return;
} /* alert_check_overflow_file */


/* =========================================================
 * alert_retrive_from_overflow:
 * Called when an alert delete was successfull. We should
 * have a spare memory slot if an entry was deleted, so
 * if we have any buffered entries read the next in.
   ========================================================= */
void alert_retrieve_from_overflow( void ) {
  long records_buffered;
  int  iostatus, memory_index, bytesread;
  message_data_def disk_record;

#ifdef DEBUG_ON
  myprintf( "DEBUG: Checking if alerts to be retrieved from overflow file\n" );
  myprintf( "DEBUG: Last in use table entry is %d, can grow to %d\n", last_table_entry, MAX_ASRV_MEM_TABLESIZE );
#endif

  records_buffered = alerts_num_buffered();

  /* read until either the buffer overflow file is empty or the memory table
   * is full again */
  while ((records_buffered > 0) && (last_table_entry < MAX_ASRV_MEM_TABLESIZE)) {
     iostatus = fseek( event_log_handle,
                      (sizeof(message_data_def) * next_evt_record_to_read), SEEK_SET );
     if (iostatus == 0) {
        bytesread = fread( (char *)&disk_record, sizeof( message_data_def ),
                           1, event_log_handle );
        if (bytesread != 1) { /* not one element read */
           /* corrupt record, or no records ! */
           myprintf( "WARN: Unable to read records from buffer overflow file, flushing overflow file\n" );
           alerts_recreate_overflow_file();
		   records_buffered = 0; /* obviously none anymore */
        }
        else {
           /* OK, we read one, process as normal */
#ifdef DEBUG_ON
           myprintf( "DEBUG: Retrieved alert for %s %s\n",
                     disk_record.msg_header.hostname, disk_record.msg_header.unique_key );
#endif
           process_alert_message( (message_data_def *)&disk_record, 1 );
           next_evt_record_to_read++;
        }
        records_buffered--;
     } /* if iostatus == 0 */
     else {
        myprintf( "WARN: Unable to read records from buffer overflow file, flushing overflow file\n" );
        alerts_recreate_overflow_file();
        records_buffered = 0; /* obviously none anymore */
     }
  } /* while */

  // BUGFIX: Somehere in the logic we never seemed to calculate that
  // we had cleared the buffer file. Do an additional check here.
  // If we reach the end of the log it's a candidate to clear.
  // ACTUALLY, bug appears to be in not reading the last few records
  // out of the file, fixed seperately. Left this in as a safety net.
  if (feof(event_log_handle)) {
     alerts_recreate_overflow_file();
  }
} /* alert_retrieve_from_overflow */


/* =========================================================
 * process_alert_message:
   ========================================================= */
void process_alert_message( message_data_def *buf, int fromOverflow ) {
  int i;

  /* ensure the key lookup fields are the correct size before doing
   * anything, some of the console apps didn't pad them out. */
  while (strlen(buf->msg_header.unique_key) < MAX_ASRV_KEY_LEN) { strcat(buf->msg_header.unique_key, " "); }
  while (strlen(buf->msg_header.hostname) < MAX_ASRV_HOSTNAME_LEN) { strcat(buf->msg_header.hostname, " "); }

  /* lets see what we have */
  if ( (memcmp(buf->msg_header.messagetype, "AC", 2) == 0) ||
       (memcmp(buf->msg_header.messagetype, "AW", 2) == 0) ||
       (memcmp(buf->msg_header.messagetype, "IO", 2) == 0) )
  {
     alert_add( buf );
  }
  else if (memcmp(buf->msg_header.messagetype, "AK", 2) == 0)
  {
     i = alert_delete( buf, fromOverflow );
     if ( i > 0 ) {  /* i memory entries were released */
        alert_retrieve_from_overflow();
        alert_check_overflow_file(); /* have we cleared the file ? */
     }
  }
  else if (memcmp(buf->msg_header.messagetype, "KA", 2) == 0)
  {
     alert_acknowledge( buf, fromOverflow );
  }
  else if (memcmp(buf->msg_header.messagetype, "KH", 2) == 0)
  {
     alert_delete_all_for_host( buf, fromOverflow );
  }
  else if (memcmp(buf->msg_header.messagetype, "LO", 2) == 0)
  {
      /* log only, already done at the start of the procedure */
  }
  /* Else an unknown type, discard it for now */
  else {
      /* log it, but do nothing */
      myprintf( "*ERR: Unknown message type %c%c from %s, discarded\n",
                buf->msg_header.messagetype,
                buf->msg_header.messagetype[1],
                buf->msg_header.hostname );
  }
  return;
} /* process_alert_message */


/* =========================================================
 * process_alert_request:
   ========================================================= */
void process_alert_request( FILE *tx, message_data_def *buf ) {
  int z;
  message_data_def *reply_buf;

  /* record each alert recieved */
  alert_record_event( buf );

#ifdef DEBUG_ON
  myprintf( "DEBUG:process_alert_request, recording below event...\n" );
#endif
#ifdef DEBUG_HEADERS
  dump_alert_buffer_to_log( buf );
#endif

  /* most alert forwarding services don't expect a reply, but need to check */
  if (buf->msg_header.expect_reply == 'Y') {
    /* If they do, all we do at the moment is reply with an ACK buffer,
       so lets create and populate one, and send it
    */
    if ( (reply_buf = malloc(sizeof(message_data_def))) != NULL ) {
      strncpy( reply_buf->msg_header.productid, 
               buf->msg_header.productid,
               sizeof(common_header_def) );
      strcpy( reply_buf->msg_header.messagetype, "--" );  /* ack message */
      reply_buf->datapersists = 'N';
      zero_data_field( reply_buf->datastr, MAX_ASRV_REMOTEHOST_DATA_LEN );
      number_to_string_with_null( (char *)&reply_buf->datastrlen, 0, MAX_ASRV_LEN_STR+1 );
      zero_data_field( reply_buf->optdata, MAX_ASRV_OPT_DATA );
	  pad_header_fields( (common_header_def *)reply_buf );
      reply_buf->NL = '\n';
      z = fwrite( reply_buf, sizeof(message_data_def), 1, tx ); 
      if ( z == -1 ) {
         myprintf(  "*ERR: Unable to ACK message from host %s\n",
                     buf->msg_header.hostname );
      }
      free( reply_buf );
    }
    else {
       myprintf( "*ERR: Insufficient server memory to ACK message from %s\n",
                 buf->msg_header.hostname );
    }
  } /* end of reply expected */

  process_alert_message( buf, 0 );
  return;
} /* process_alert_request */


/* *********************************************************
   Console program interface routines.
   ********************************************************* */

/* =========================================================
 * console_log_event:
 * each event recieved is logged. As such we also need to
 * record any console command that will modify the event.
   ========================================================= */
void console_log_event( console_io_data_def *buf ) {
   int  linelen;
   char msgbuffer[129]; /* 128 + null */

   linelen = snprintf( msgbuffer, 128,
             "AINF: %s %s %c%c %s (CONSOLE COMMAND)",
             buf->msg_header.hostname,
             buf->msg_header.source_date_time,
             buf->msg_header.messagetype[0],
             buf->msg_header.messagetype[1],
             buf->msg_header.unique_key );
   msgbuffer[128] = '\0';  /* in case too much data for a null to be added */
   myprintf( "%s\n", msgbuffer );
   return;
} /* console_log_event */


/* =========================================================
 * alert_deletelist:
 * Issued from a console app when the user on the console
 * wants to delete all alerts down to the last one on their
 * display screen. The key value of the last entry on the
 * users screen will be as hostname/key in the message header.
   ========================================================= */
void alert_deletelist( console_io_data_def *buf ) {
   int foundpos, i;

   if (last_table_entry <= 0) { return; } /* nothing to do */
   foundpos = alert_lookup( (common_header_def *)buf );
   if (foundpos < 0) { return; } /* nothing to do */

   /* Set all entries up to the one found to deleted */
   for (i = 0; i <= foundpos; i++) {
      memory_table[i].msg_header.messagetype[0] = 'D';
      memory_table[i].msg_header.messagetype[1] = 'D';
      myprintf( "AINF: Manual delete of %s %s (CONSOLE KK)\n",
                memory_table[i].msg_header.hostname,
                memory_table[i].msg_header.unique_key );
   }

   /* Compress the momory table */
   i = alerts_compress();
   myprintf( "AINF: %d alerts KK deleted, %d recovered by compress\n",
             (foundpos + 1), i );

   /* if any were removed from memory see if we can load some
	* more from the overflow file */
   if ( i > 0 ) {  /* i memory entries were released */
      alert_retrieve_from_overflow();
      alert_check_overflow_file(); /* have we cleared the file ? */
   }

   /* all done */
   return;
} /* alert_deletelist */


/* =========================================================
 * write_console_reply_buffer:
   ========================================================= */
void write_console_reply_buffer( FILE *tx, console_io_data_def *buf ) {
   int z;
   char last_flag;

   /* we don't want a reply from the console ever, but we need
	* to keep the console reply flag, if we leave it set to N
	* then the mainline will never call us for the final write. */
   last_flag = buf->msg_header.expect_reply;
   buf->msg_header.expect_reply = 'N';

   set_datestamp( (char *)&buf->msg_header.source_date_time );
   number_to_string_with_null( (char *)&buf->datastrlen, strlen(buf->datastr), MAX_ASRV_LEN_STR+1 );
   translate_nl_to_hat( buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
   pad_header_fields( (common_header_def *)buf );
   buf->NL = '\n';
   z = fwrite( buf, sizeof(console_io_data_def), 1, tx ); 
   if ( z == -1 ) {
      myprintf("*ERR: Unable to write a response to a client console application\n");
   }
#ifdef DEBUG_ON
   myprintf( "DEBUG: write_console_reply_byffer\n" );
   myprintf( "DEBUG: Wrote %d records to console client, expected to write 1 record (%d bytes)...\n", z,
             sizeof(console_io_data_def));
#endif
#ifdef DEBUG_HEADERS
   dump_console_buffer_to_log( buf );
#endif

   buf->msg_header.expect_reply = last_flag;
   return;
} /* write_console_reply_buffer */


/* =========================================================
 * add_to_console_io_buffer:
   ========================================================= */
void add_to_console_io_buffer( FILE *tx, console_io_data_def *buf, char *dataline, int datalinelen ) {
   int buf_len, z;

   buf_len = strlen(buf->datastr);

   /* it it won't fit in this buffer, flush current buffer and start another */
   if ((buf_len + datalinelen) >= MAX_ASRV_CONSOLE_DATA_LEN) {
      /* set len */
      buf->msg_header.messagetype[0] = 'D'; /* display */
      buf->msg_header.messagetype[1] = 'F'; /* more will follow */
      write_console_reply_buffer( tx, buf );
      zero_data_field( buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
      number_to_string_with_null( (char *)&buf->datastrlen, 0, MAX_ASRV_LEN_STR+1 );
   }

   /* add to the old/new buffer */
   strcat( buf->datastr, dataline );

   return; 
} /* add_to_console_io_buffer */


/* =========================================================
 * console_command_to_remote:
   ========================================================= */
void console_command_to_remote( void ) {
   /* No plans to implement this yet */
   return;
} /* console_command_to_remote */


/* =========================================================
 * console_get_details:
   ========================================================= */
void console_get_details( FILE *tx, console_io_data_def *buf ) {
   int index;

   /* look up the alert details */
   if ( (index = alert_lookup( (common_header_def *)buf )) < 0) {
      /* Failed to find it */
      strcpy( buf->msg_header.messagetype, "DE" );
      sprintf( buf->datastr, "%s", "*ERR: Alert no longer exists\n" );
      number_to_string_with_null( (char *)&buf->datastrlen, 29, MAX_ASRV_LEN_STR+1 );
      return;  /* mainline will write it out */
   }
   
   /* format into output buffer */
   snprintf( buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN,
             "HOST: %s   DATETIME: %s\nKEY: %s\nMSGTYPE %c%c     PERSISTENT %c\nMSGTEXT: %s\nOPTIONAL DATA: %s\n",
             memory_table[index].msg_header.hostname,
             memory_table[index].msg_header.source_date_time,
             memory_table[index].msg_header.unique_key,
             memory_table[index].msg_header.messagetype[0],
             memory_table[index].msg_header.messagetype[1],
             memory_table[index].datapersists,
             memory_table[index].datastr,
             memory_table[index].optdata );
   strcpy( buf->msg_header.messagetype, "DS" );   /* DS to display details */

   /* The main proc will write it out */  
   return;
} /* console_get_details */


/* =========================================================
 * console_refresh_me:
 * Invoked when a console application sends a request to
 * refresh its display buffer.
   ========================================================= */
void console_refresh_me( FILE *tx, console_io_data_def *buf ) {
   int i, console_lines, linelen, max_line_len;
   char workbuffer[ASRV_MAX_CONSOLE_LINE_LEN+1];

#ifdef DEBUG_ON
   myprintf( "DEBUG: in console_refresh_me\n" );
#endif

   /* 1. Create a buffer to hold part of the response 
      2. format as much data as we can into the buffer
      3. send data to client
      4. loop back to 2 while data remains to be still sent
   */

   /* set a default */
   max_line_len = ASRV_MAX_CONSOLE_LINE_LEN;

#ifdef DEBUG_ON
   myprintf( "DEBUG: last_table_entry = %d...\n", last_table_entry );
#endif

  /* build up the entire formatted display, use line info provided by client if
   * possible to do so, check it for sanity though. */
  console_lines = message_from_console_application( (common_header_def *)buf,
                                                    (int *)&max_line_len );
  if (console_lines > last_table_entry) { console_lines = last_table_entry; }
  if (max_line_len > ASRV_MAX_CONSOLE_LINE_LEN) { max_line_len = ASRV_MAX_CONSOLE_LINE_LEN; }
#ifdef DEBUG_ON
   myprintf( "DEBUG: adjusted console_lines = %d...\n", console_lines );
#endif
  for ( i = 0; i < console_lines; i++ ) {
     linelen = snprintf( (char *)&workbuffer, max_line_len, "%s %s %s %c%c %s\n",
             memory_table[i].msg_header.source_date_time,
             memory_table[i].msg_header.hostname,
             memory_table[i].msg_header.unique_key,
             memory_table[i].msg_header.messagetype[0],
             memory_table[i].msg_header.messagetype[1],
             memory_table[i].datastr );
     /* if we had a full line no lf or null was added so force them in */
     if (linelen >= max_line_len) { /* was a full line, no LF added so force one */
        workbuffer[max_line_len-1] = '\n';
        workbuffer[max_line_len] = '\0';  /* we allowed +1 in buffer for this */
        linelen = max_line_len;
#ifdef DEBUG_ON
        myprintf("DEBUG: console line len >= %d, truncated to %s\n", max_line_len, workbuffer );
#endif
     }
     /* the add proc will write the buffers as they fill automatically */
     add_to_console_io_buffer( tx, buf, (char *)&workbuffer, linelen ); 
  }

  /* If there are no alerts lets show an info only message */
   if (last_table_entry == 0) {
#ifdef DEBUG_ON
      myprintf( "DEBUG: Writing the NO ALERTS message\n" );
#endif
      set_datestamp( (char *)&buf->datastr );
      strcat( buf->datastr, " --------------- -------------- IO NO ALERTS AT PRESENT\n" );
   }
#ifdef DEBUG_ON
   else {
      myprintf( "DEBUG: Alerts were found, so returning alerts to console\n" );
   }
#endif

  /* now we just need to write the last buffer, the caller does that,
     we just need to ensure the correct fields are set. */
   buf->msg_header.messagetype[0] = 'D';
   buf->msg_header.messagetype[1] = 'B';
   return;
} /* console_refresh_me */

/* =========================================================
 * process_console_request:
   ========================================================= */
void process_console_request( FILE *tx, console_io_data_def *buf ) {
  int z;
  console_io_data_def *reply_buf;

#ifdef DEBUG_HEADERS
  myprintf( "DEBUG: process_console_request, Data recieved...\n" );
  dump_console_buffer_to_log( buf );
#endif

  /* Create a reply buffer, to avoid overwriting command */
  if ( (reply_buf = malloc(sizeof(console_io_data_def))) == NULL )
  {
     myprintf( "%s", "*ERR: Insufficent server memory for console request\n" );
     if (buf->msg_header.expect_reply != 'Y') {
        return;
     }
     /* have to use existing buffer for the reply */
     buf->msg_header.messagetype[0] = 'D';
     buf->msg_header.messagetype[1] = 'E';
     sprintf( buf->datastr, "%s", "*ERR: Insufficent server memory for request\n" );
     /* write the reply */
     write_console_reply_buffer( tx, buf );

     /* done */
     return;
  }

  /* set the new buffer defaults */
  memcpy( reply_buf->msg_header.productid, buf->msg_header.productid,
          sizeof(common_header_def) );
  reply_buf->msg_header.messagetype[0] = 'D';
  reply_buf->msg_header.messagetype[1] = 'B';
  number_to_string_with_null( (char *)&reply_buf->datastrlen, 0, MAX_ASRV_LEN_STR+1 );
  zero_data_field( (char *)&reply_buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
  buf->NL = '\n';     

  /* Lets see what the command was */
  /* We can have one of...
     RC=remote command  (not yet implemented)
     AC=acknowledge alert
     AD=delete alert
     SL=show list (produce alert list for client)
     SD=show details (for selected alert)
     KK=delete number of alerts defined by datastr
  */
  if (memcmp(buf->msg_header.messagetype,"SL",2) == 0)
  {
     if (buf->msg_header.expect_reply == 'Y') {
        console_refresh_me( tx, reply_buf );
     }
     /* else no point as we won't send anything */
  }
  else if (memcmp(buf->msg_header.messagetype,"SD",2) == 0)
  {
     if (buf->msg_header.expect_reply == 'Y') {
        console_get_details( tx, reply_buf );
     }
     /* else no point as we won't send anything */
  }
  else if (memcmp(buf->msg_header.messagetype,"AD",2) == 0)
  {
     console_log_event( buf );  /* alert state change to be recorded */
     z = alert_delete( (message_data_def *)buf, 0 );
     if ( z > 0 ) {  /* z memory entries were released */
        alert_retrieve_from_overflow();
        alert_check_overflow_file(); /* have we cleared the file ? */
     }
     if (buf->msg_header.expect_reply == 'Y') { /* format an ACK message */
        reply_buf->msg_header.messagetype[0] = '-';
        reply_buf->msg_header.messagetype[1] = '-';
     }
  }
  else if (memcmp(buf->msg_header.messagetype,"AC",2) == 0)
  {
     console_log_event( buf );  /* alert state change to be recorded */
     alert_acknowledge( (message_data_def *)buf, 0 );
     if (buf->msg_header.expect_reply == 'Y') { /* format an ACK message */
        reply_buf->msg_header.messagetype[0] = '-';
        reply_buf->msg_header.messagetype[1] = '-';
     }
  }
  else if (memcmp(buf->msg_header.messagetype,"KK",2) == 0)
  {
     alert_deletelist( buf );
     if (buf->msg_header.expect_reply == 'Y') { /* format an ACK message */
        reply_buf->msg_header.messagetype[0] = '-';
        reply_buf->msg_header.messagetype[1] = '-';
     }
  }
  else if (memcmp(buf->msg_header.messagetype,"RC",2) == 0)
  {
     /* Not implemented */
     if (buf->msg_header.expect_reply == 'Y') { /* format an error message */
        reply_buf->msg_header.messagetype[0] = 'D';
        reply_buf->msg_header.messagetype[1] = 'E';
		strcpy( reply_buf->datastr, "Server does not have that command implemented yet !\n" );
     }
  }
  else {
     /* Invalid request, ignore it */
     if (buf->msg_header.expect_reply == 'Y') { /* format an error message */
        reply_buf->msg_header.messagetype[0] = 'D';
        reply_buf->msg_header.messagetype[1] = 'E';
		strcpy( reply_buf->datastr, "Server does not recognise command, ignored !\n" );
     }
     myprintf( "*ERR: Unknown command '%c%c' recieved from console at %s, ignored\n",
               buf->msg_header.messagetype[0],
               buf->msg_header.messagetype[1],
               buf->msg_header.hostname );
  }

  /* Consoles may not always want a reply, so check */
  if (buf->msg_header.expect_reply != 'Y') {
     /* Done */
     free(reply_buf);
     return;
  }

  /* write the reply from reply_buf */
  write_console_reply_buffer( tx, reply_buf );

  /* Done */
  free(reply_buf);
  return;
} /* process_console_request */


/* =========================================================
 * process_client:
 * 
   ========================================================= */
static int process_client( int sock_id, int *shutdown_flag ) {
   common_header_def *header_block;
   char buf[MAX_ASRV_IO_BUFFSIZE+1];   /* io buffer, static as called all the time,
                                     so malloc would just be an overhead */
   FILE *rx = client[sock_id].rx;
   FILE *tx = client[sock_id].tx;
   int zz, i, junk;

   /* select the correct stack */
   stack = (char *) client[sock_id].stack;
   sp = client[sock_id].sp;

   /* if not EOF of stack process one line */
   if (!feof(rx) && (fgets(buf,sizeof buf,rx) != NULL) ) {
       /* a small trap, for some reason an empty second buffer is read from console
        * applications after the first has been accepted ?. */
      zz = strlen(buf);
      if (zz < 1) {
         myprintf( "0 byte data buffer recieved, ignored\n" );
         return 0;
      }  /* no data, ignore but return 0 to carry on */

      header_block = (common_header_def *)&buf;
#ifdef DEBUG_HEADERS
      myprintf( "DEBUG: process_client, About to process below buffer...\n" );
      dump_header_to_log( header_block );
#endif
      /* is it a known alert forwarding application ? */
      if (message_from_forwarding_application(header_block) == 1)
      {
          client[sock_id].connect_service = 'F'; /* alert forwarder */
          process_alert_request( tx, (message_data_def *)buf );
      }
      /* Else is it a known console app */
      else if (message_from_console_application(header_block,(int *)&junk) > 0)
      {
          client[sock_id].connect_service = 'C'; /* console app */
          process_console_request( tx, (console_io_data_def *)buf );
      }
      /* Else is it a known kill server app */
      else if ( memcmp(header_block->productid, "KILLER", 6) == 0 ) /* supplied shutdown proc */
      {
          myprintf( "INFO: Shutdown request recieved !\n" );
          shutdown_immed( 0 );
      }
      /* Else deny, force client off */
      else {
		zz = strlen(buf);
        myprintf( "WARN: Unknown app, ignoring data from %s, datalen=%d\n",
                  client[sock_id].remoteaddr, zz );
		for (i = 0; i < zz; i++) {
           if ( (buf[i] < ' ') || (buf[i] > '~') ) { buf[i] = '.'; }
		}
		myprintf( "Ignored data=%s\n", buf );
		/*
        myprintf( "WARN: Unknown app, forcing disconnect from %s\n",
                  client[sock_id].remoteaddr );
        fclose( tx );
        shutdown(fileno(rx),SHUT_RDWR);
        fclose( rx );
        client[sock_id].rx = client[sock_id].tx = NULL;
        while (sp > 0) {
           free( &stack[--sp] );
        }
        free(stack);
        client[sock_id].stack = NULL;
        client[sock_id].sp = 0;
        return EOF;
		*/
      }
   }

   /* is still stuff stacked just return */
   if (!feof(rx) ) {
#ifdef DEBUG_ON
      myprintf(  "DEBUG: process_client, Not EOF on data stream, so saving stack pointer and returning...\n" );
#endif
      client[sock_id].sp = sp;
      return 0;
   }

   /* this could be a normal exit/disconnect from the service so don't treat as
    * an error !. */
   /* if we get here the client sent an EOF so we are done with it */
   fclose( tx );
   shutdown(fileno(rx),SHUT_RDWR);
   fclose( rx );

#ifdef DEBUG_ON
   myprintf(  "DEBUG: Disconnect received from %s\n", client[sock_id].remoteaddr );
#endif

   client[sock_id].rx = client[sock_id].tx = NULL;
   while (sp > 0) {
      free( &stack[--sp] );
   }
   free(stack);
   client[sock_id].stack = NULL;
   client[sock_id].sp = 0;

   return EOF;
} /* process_client */


/* =========================================================
 * main:
 * 
 * Initialise all the function libraries.
 * If OK establish a socket connection for the port we listen for requests on.
 * Loop waiting for client activity.
 *   On client activity process the client request.
 *   On inactivity timer see if we have any other work to do before listening
 *   again.
   ========================================================= */
int main (int argc, char **argv, char **envp) {
   int z;
   int use_specific_interface = 0;  /* default is bind to any interface */
   struct sockaddr_in adr_inet;    /* AF_INET */
   int len_inet;                   /* length */

   struct sockaddr_in connect_addr;    /* AF_INET */
   int listen_sock;
   int connect_addr_len;
   int connect_sock;      /* incomming connect socket file handle */
   struct utsname u_name;
   int srvr_port;         /* port number to use */

   /* for getsockettest */
   char buf[64];

   /* for writes */
   char workbuf[128];
   int  workbufdatalen;
   time_t td;

   fd_set rx_set;      /* read set */
   fd_set work_set;    /* work set */
#ifdef SOLARIS
   /* SOLARIS timestamp value */
   struct timespec tv;   /* timeout value SOLARIS */
#else
   /* LINUX assumed */
   struct timeval tv;  /* timeout value LINUX */
#endif
   int mx;   /* Max fd + 1 */
   int n;    /* socket/file activity */

   long status_flag;
   int job_status;
   long file_check_counter;
   
   /* determines if a client evere asks us to leave the main loop and terminate */
   int shutdown_flag;
   
   /* Before we do anything check our buffer sizes, we need to
      make sure any enhancements I have added have not caused
      any overrun possibilities */
   if (
         (sizeof(message_data_def) > MAX_ASRV_IO_BUFFSIZE) ||
         (sizeof(console_io_data_def) > MAX_ASRV_IO_BUFFSIZE)
      )
   {
       die( "Programmer error: buffer is too small", DIE_INTERNAL );
   }

   /* check for option overrides of the default port 9003 */
   if (argc >= 2 ) {
      srvr_port = atoi(argv[1]);
   }
   else {
      srvr_port = 9003;
   }
   /* there may be a requirement to bind to one interface only */
   if (argc >= 3) {
      use_specific_interface = 1; /* use whats in argv[2] */
   }

   /* We should open our log file first, so we can actually use it */
   SERVER_roll_log_files();
   /* Open the event overflow file now */
   next_evt_record_to_read = 0;
   if ( (event_log_handle = fopen( event_logfile_name, "a+" )) == NULL ) {
      myprintf( "*ERR: Unable to open alert overflow buffer file %s, aborting!\n", event_logfile_name );
      die( "IO error on alert overflow file", DIE_INTERNAL );
   }

   /* Then startup the server */
   myprintf(  "----: Alert collections server - Copyright (C) Mark Dickinson, 2003, all rights reserved\n" );
   myprintf(  "----: This version %s is provided under the GPL license,\n---- it comes with no warantee of suitability for your site,\n", application_version );
   myprintf(  "----: you run this application at your own risk.\n" );
   myprintf(  "----: Server initialisation beginning\n" );
   shutdown_flag = 0;     /* of course we don't want to shutdown yet */
   last_table_entry = 0;  /* no alerts in memory yet */
   unlink_permitted = 0;

   /*
    * TCPIP stuff 
    * Initialise the client table we use to manage sessions. Open a socket
    * session to tcp-ip services and listen on the configured port. Indicate
    * read interest on the socket.
    */

   /* Initialise client structure */
   for ( z = 0; z < MAX_ASRV_CLIENTS; ++z ) {
      client[z].stack = NULL;
      client[z].sp = 0;
      client[z].rx = NULL;
      client[z].tx = NULL;
      client[z].remoteport = 0;
      client[z].accept_unsol = 'N';
      client[z].connect_service = '-';
      strcpy(client[z].remoteaddr,"SPARE");
   }

   /* IPv4 internet socket */
   listen_sock = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
   if ( listen_sock == -1 ) die( "socket()", DIE_TCPIP_SETUP );

   /* create an inet address */
   memset( &adr_inet, 0, sizeof adr_inet );
   adr_inet.sin_family = AF_INET;
   adr_inet.sin_port = htons(srvr_port);

   /* Check to see if we allow binding on any local 
	* interface, or if we are retricted to a single
	* interface */
   if (use_specific_interface != 0) {
      /* bind only to the address provided */
      myprintf( "INFO: Server configured to bind to interface %s, port %d only\n", argv[2], srvr_port );
      adr_inet.sin_addr.s_addr = inet_addr(argv[2]);
   }
   else {
      /* ---- allow binding on any local interface ---- */
      /* (also allows to connect out via any interface) */
      myprintf( "INFO: Server configured to bind to all interfaces, port %d\n", srvr_port );
      adr_inet.sin_addr.s_addr = htonl(INADDR_ANY);
   }

#ifdef SOLARIS
   /* Solaris does not have a INADDR_NONE defined
    * but returns -1 for a bad address */
   if (adr_inet.sin_addr.s_addr == -1 ) {
#else
   if ( adr_inet.sin_addr.s_addr == INADDR_NONE ) {
#endif
      die( "Translating IP address", DIE_TCPIP_SETUP );
   }

   len_inet = sizeof adr_inet;

   /* bind the address to the socket */
   z = bind( listen_sock, (struct sockaddr *)&adr_inet, len_inet );
   if ( z == -1 ) {
      myprintf( "*ERR: Unable to bind to configured interface(s)\n" );
      snprintf( workbuf, 127, "bind() port %d", srvr_port );
      die(workbuf, DIE_TCPIP_SETUP); 
   }

   /* listen on the port looking for connections */
   z = listen( listen_sock, 10 );  /* allow up to 10 connect requests to be queued */
   if ( z == -1 ) die("listen()", DIE_TCPIP_SETUP );

   /* pass the socket around to ensure we can get port info */
   if ( !get_sock_addr(listen_sock,buf,sizeof buf) ) die("get_sock_addr", DIE_TCPIP_SETUP );
   myprintf(  "INFO: Server listening on port address '%s'\n", buf );

   /* Express interest in the socket file for READ events */
   FD_ZERO( &rx_set );
   FD_SET( listen_sock, &rx_set );
   mx = listen_sock + 1;   /* max fd + 1 */

   myprintf(  "INFO: TCP-IP Services now available\n" );

#ifdef SOLARIS
   myprintf(  "INFO: Server has initialised\n" );
#else
   myprintf(  "INFO: Server has initialised, going daemon now\n" );
   /* Switch ourselves to run as a deamon now */
   z = daemon( 1, 1 ); /* don't change directory to /, dont redirect stderr and stdout to /dev/null */
   if (z == -1) {
      perror( "Request to run as a daemon" );
      myprintf(  "*ERR: Unable to run as a deamon task, running as a normal task !\n" );
   }
#endif

   /* start a server loop, for is more efficient than checking shutdown_flag each loop */
   for (;;) {
      /* copy the rx set to the work set */
      FD_ZERO( &work_set );
      for ( z = 0; z < mx; ++z ) {
         if ( FD_ISSET(z,&rx_set) ) FD_SET( z, &work_set );
      }

      /* set a timeout of 1.3 seconds, and wait for activity */
      tv.tv_sec = 1;
#ifdef SOLARIS
      /* Solaris generates a warning here, I don't know why ? */
#endif
      tv.tv_usec = 30000;
      n = select( mx, &work_set, NULL, NULL, &tv );
      if ( n == -1 ) {
         die( "select()", DIE_TCPIP_SETUP );
      }

      /* ---------------------------------------------------------------------
       *  Our 1.3 second timer expired without any activity.
       *  We use this inactivity trigger to check the scheduler activity tasks
       *  to see if there is anything we need to do; run jobs, start newday
       *  etc.
       * ----------------------------------------------------------------------*/
      else if ( !n ) {
         /* --- timer expired, check scheduler stuff */
         /* --- FIRST (new) see if we are pending a shutdown, and if we can */
         if ( shutdown_flag == 1) {
               shutdown(listen_sock,SHUT_RDWR);
               close( listen_sock );
               shutdown_immed( 0 );
         }
         continue;
      }

      /* -------------------------------------------------------------------
       * We will always check to see if any new clients are trying to connect.
       * ------------------------------------------------------------------- */
      /* check if a connect has occurred */
      if ( FD_ISSET( listen_sock, &work_set ) ) {
         /* wait for a connect */
         connect_addr_len = sizeof connect_addr;
         connect_sock = accept( listen_sock, (struct sockaddr *)&connect_addr,
                                &connect_addr_len );
         if ( connect_sock == -1 ) die("accept()", DIE_TCPIP_SETUP );

         /* check to ensure we havn't exceeded capacity */
         if ( connect_sock > MAX_ASRV_CLIENTS ) {
            myprintf(  "WARN: Client connection refused, server alrady has maximum connections allowed !\n" );
            close( connect_sock );   /* at capacity */
            continue;
         }

         /* if we have a connection  we need to open read and write streams on
            that socket connection */
         client[connect_sock].rx = fdopen( connect_sock, "r" );
         if ( client[connect_sock].rx == NULL ) die( "fdopen()", DIE_TCPIP_SETUP );
         client[connect_sock].tx = fdopen(dup(connect_sock), "w" );
         if ( client[connect_sock].tx == NULL ) {
            fclose( client[connect_sock].rx );
            die( "fdopen( dup() )", DIE_TCPIP_SETUP );
         }
         if ( connect_sock + 1 > mx ) mx = connect_sock + 1;

         /* set both streams to line buffered mode */
         setlinebuf( client[connect_sock].rx );
         setlinebuf( client[connect_sock].tx );

         /* allocate a stack */
         client[connect_sock].sp = 0;
         client[connect_sock].stack = (mpz_t **) malloc(sizeof (mpz_t *) * MAX_ASRV_STACK);
#ifdef DEBUG_ON
         myprintf(  "DEBUG: Allocating stack memory for new connection\n" );
#endif
         FD_SET( connect_sock, &rx_set );

         /* do work  */
         /* log the connection we have received */
         if (get_remote_addr( connect_sock, (char *) client[connect_sock].remoteaddr, 50 ) != NULL) {
#ifdef DEBUG_ON
            myprintf(  "DEBUG: Connection accepted from %s\n", client[connect_sock].remoteaddr );
#endif
         }
         else {
            myprintf(  "INFO: Connection from a host that hides its ip address\n" );
            strcpy( client[connect_sock].remoteaddr, "Hidden\n\0" );
         }
         z = uname( &u_name );
         if ( z == -1 ) {
            myprintf("WARN: uname error\n" );
            strcpy( u_name.nodename, "UN-NAMED HOST" );
         }
      }  /* end if FD_ISSET */            

      /* -------------------------------------------------------------------
       *  And we will always check to see if any clients have asked us to do
       * anything.
       * ------------------------------------------------------------------- */
      /* check for client activity */
      for ( connect_sock = 0; connect_sock < mx; ++connect_sock ) {
         if ( connect_sock == listen_sock ) 
            continue;      /* don't worry about listen_sock */
         if ( FD_ISSET(connect_sock, &work_set) ) {
            if ( process_client(connect_sock, &shutdown_flag) == EOF ) {
               FD_CLR(connect_sock, &rx_set);
            }
            /* Check if the client issued a shutdown request that we accepted */
            if (shutdown_flag == 1) {
               shutdown(listen_sock,SHUT_RDWR);
               close( listen_sock );
               myprintf(  "INFO: Server is now no longer acception connections...\n" );
               shutdown_immed( 0 );
            }
         }
      }
   }  /* end of for (;;) */

   /* should never get here, but manage anyway */

   /* clean up and exit */
   shutdown(listen_sock,SHUT_RDWR);
   close( listen_sock );
   myprintf(  "*ERR: SHUTTING DOWN DUE TO UNEXPECTED MAIN TERMINATION\n" );
   shutdown_immed( 0 );
   return 0;
}  /* main */
