/*
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
*/

/*
 * log_parser.c
 *
 * This is designed to help with the raising and cancelling of alerts
 * with my automation tools.
 *
 * It may also be usefull for others, who don't use my tools.
 *
 * Basicaly it is designd to process the output of log files, and use
 * a ruleset to decide whether a log message is to trigger an alert
 * or cancel an alert.
 *
 * The command syntax would ideally be...
 *   tail -f <yourlogfile> | log_parser <parm-file> | <yourscript>
 *
 * The parm file is expected to contain entries of...
 *   <action> <key> <datastartoffset> <text-string>
 *
 * MID: 16Jan2004 added support for entry in parm file of form
 *   USEHOSTNAME=nn
 * See change history below for use, purpose and effects on
 * the documented output.
 *
 *  1. action is either A (raise alert) or X (cancel alert alert). It may
 *     also be from "1" to "5" for user custom handling.
 *  2. key should be unique for the application as it is used to both raise
 *     and cancel alerts for the event.
 *  3. datastartoffset indicates at what point in the message you want the
 *     message text to begin, this is to help you skip any timestamps etc.
 *     that may be in the message string, that you don't want forwarded on.
 *  4. text-string is the text that will be searched for within the message
 *     piped in from the log file.
 *
 *  The A/X raise/cancel alerts should be grouped together in the parm file
 *  for readability. An alert cancel works on an alert key so it you have
 *  5 events for one program that can raise an alert, and 3 events that
 *  can cancel an alert, thet should all have the same key, so a cancel
 *  when the program is 'all-ok' will cancel any of the outstanding alerts.
 *
 *  Sample: Samba fighting for control...
 *  /var/log/messages has
 *  Feb 28 22:50:39 falcon nmbd[2713]: [2003/02/28 22:50:39, 0]
 *  nmbd/nmbd_incomingdgrams.c:process_local_master_announce(307) 
 *  Feb 28 22:50:39 falcon nmbd[2713]:   process_local_master_announce: Server
 *  XP-HOMEPC at IP 169.254.218.189 is announcing itself as a
 *   local master browser for workgroup WORKGROUP and we think we are master.
 *   Forcing election. 
 *   Feb 28 22:50:39 falcon nmbd[2713]: [2003/02/28 22:50:39, 0]
 *   nmbd/nmbd_become_lmb.c:unbecome_local_master_success(155) 
 *   Feb 28 22:50:39 falcon nmbd[2713]:   ***** 
 *   Feb 28 22:50:39 falcon nmbd[2713]:    
 *   Feb 28 22:50:39 falcon nmbd[2713]:   Samba name server FALCON has stopped
 *   being a local master browser for workgroup WORKGROUP on subnet 169.254.218.183 
 *   Feb 28 22:50:39 falcon nmbd[2713]:    
 *   Feb 28 22:50:39 falcon nmbd[2713]:   ***** 
 *   Feb 28 22:50:56 falcon nmbd[2713]: [2003/02/28 22:50:56, 0]
 *   nmbd/nmbd_become_lmb.c:become_local_master_stage2(405) 
 *   Feb 28 22:50:56 falcon nmbd[2713]:   ***** 
 *   Feb 28 22:50:56 falcon nmbd[2713]:    
 *   Feb 28 22:50:56 falcon nmbd[2713]:   Samba name server FALCON is now a
 *   local master browser for workgroup WORKGROUP on subnet 169.254.218.183 
 *
 *   The config file has
 *   A          SAMBA          35         Forcing election.
 *   X          SAMBA          35         now a local master browser
 *
 *   Program output from tail -f /var/log/messages | log_parser is
 *
 *    RAISE-ALERT SAMBA "  process_local_master_announce: Server XP-HOMEPC at IP 169.254.218.189 is
 *    announcing itself as a local master browser for workgroup WORKGROUP and we think we are master.
 *    Forcing election. "
 *    CANCEL-ALERT SAMBA "  Samba name server FALCON is now a local master browser for workgroup
 *    WORKGROUP on subnet 169.254.218.183 "
 *
 *
 * Change History
 * --------------
 * 16Jan2004 MID...
 * Added support for a USEHOSTNAME=nn field in the parm file read by the
 * program. If used this changes the format of the message written to
 * stdout for processing by user scripts, as detailed below. This parameter
 * was added to allow support for those sites that use a central syslog
 * server and want to just parse the one central file rather than run a parser
 * on each seperate server. With a central log file the alert to raise
 * needs to be the name of the server generating the message, not the name of the
 * server the log file is on; the nn is the field number the original hostname
 * field is to be extracted from, it will be written out in every response,
 * changing the value of the documented messages above as below...
 *    RAISE-ALERT SAMBA "  process_local_master_announce: Server XP-HOMEPC at IP 169.254.218.189 is
 *    announcing itself as a local master browser for workgroup WORKGROUP and we think we are master.
 *    Forcing election. " HOSTNAME
 *    CANCEL-ALERT SAMBA "  Samba name server FALCON is now a local master browser for workgroup
 *    WORKGROUP on subnet 169.254.218.183 " HOSTNAME
 * Where HOSTNAME will be thehostname from the message line.
 *
 * ====================================================================== */
#include <stdio.h>

#define MAX_KEY_LEN 14
#define MAX_SCAN_LEN 50
#define MAX_SCAN_TABLE_SIZE 50
#define MAX_INPUT_LINE_LEN 256

/* MID: 16Jan2004, added to support extracting the hostname value
 * the alert tools use 15, but the user may want a fuller name for
 * special processing. */
#define MAX_HOSTNAME_LEN 200

typedef struct {
		char msgtype;   /* action to take, A=alert, X=cancel */
		char alertkey[MAX_KEY_LEN+1]; /* key to be used */
		int  displaymsgdataoffset; /* if found, from what msg offset to we display */
		int  msglen; /* worked out internally, the msg scan len for memcmp */
		char msgscantext[MAX_SCAN_LEN+1]; /* the text to scan for */
} message_entry_def;

typedef struct {
    int num_entries;
    message_entry_def message_entry[MAX_SCAN_TABLE_SIZE];
} message_table_def;

message_table_def message_table;
FILE *infile;
char line_buffer[MAX_INPUT_LINE_LEN+1];

/* MID: 16Jan2004, added to support extracting the hostname value */
int usehostname_fieldnumber;
char reportinghost[MAX_HOSTNAME_LEN+1];

/* ---------------------------------------------------------------
 * store_line:
 *
 * Lines in the parm file are expected to be
 * <action> <key> <datastartoffset> <text-string>
 *
 * This proc parses the config line into the memory table
 * structure.
 *
 * MID: 16jan2004, add test for new field USEHOSTNAME=nn
 * --------------------------------------------------------------- */
int store_line( char *parmline ) {
   char *ptr, *ptr2;

   if (message_table.num_entries >= MAX_SCAN_TABLE_SIZE) {
      printf( "Error: Over %d entries in the parameter table, review table\n", MAX_SCAN_TABLE_SIZE );
      return( 0 );  /* error */
   }

   ptr = parmline;
   while (*ptr == ' ') { ptr++; } /* skip any leading spaces */
   if ( (*ptr == '\0') || (*ptr == '\n') ) {
      printf( "Error: Empty line in parameter file\n" );
      return( 0 );
   }
   if (*ptr == '#') { /* a comment line */
      return( 1 );  /* ok to continue, no actions with it to be done here */
   }

   /* MID: 16Jan2004, add test for new USEHOSTNAME=nn parameter */
   if (memcmp(ptr,"USEHOSTNAME=",12) == 0) {
      ptr = ptr + 12;  /* get the data value */
      usehostname_fieldnumber = atoi(ptr);
      /* Sanity check, it can't be > 15 fields in can it ? */
      if ( (usehostname_fieldnumber == 0) || (usehostname_fieldnumber > 15)) {
         printf( "Error: non-numeric or invalid value in USEHOSTNAME line\n" );
         return( 0 );
      }
      return( 1 );  /* ok to continue, data saved */
   }
   /* MID: 16Jan2004, end of additional test for new USEHOSTNAME=nn parameter */

   /*    alert            cancel            user range 1 through 5   */
   if ( (*ptr != 'A') && (*ptr != 'X') && ((*ptr < '1') || (*ptr > '5')) ) {  /* not legal */
      printf( "Error: Invalid action code, must be A or X. Code in file was %c\n", *ptr );
      return( 0 );  /* error */
   }
   message_table.message_entry[message_table.num_entries].msgtype = *ptr;

   /* find the next field, the key */
   while ((*ptr != ' ') && (*ptr != '\0') && (*ptr != '\n')) { ptr++; }
   if ( (*ptr == '\0') || (*ptr == '\n') ) {
      printf( "Error: Incomplete line in parameter file, no alert key found\n" );
      return( 0 );
   }
   while (*ptr == ' ') { ptr++; }
   if ( (*ptr == '\0') || (*ptr == '\n') ) {
      printf( "Error: Incomplete line in parameter file, no alert key found\n" );
      return( 0 );
   }
   ptr2 = ptr;  /* need to ensure its null terminated for our copy */
   while ((*ptr2 != ' ') && (*ptr2 != '\0') && (*ptr2 != '\n')) { ptr2++; }
   *ptr2 = '\0';
   strncpy( message_table.message_entry[message_table.num_entries].alertkey, ptr, MAX_KEY_LEN );
   message_table.message_entry[message_table.num_entries].alertkey[MAX_KEY_LEN] = '\0';
   *ptr2 = ' '; /* put it back to a space so we can show the entire data line in any
                   later error displays */
   
   /* find the next field, the datastartoffset */
   ptr = ptr2;
   ptr++;  /* skip the null we put at the end of the field (not reqd now its a space again, but leave in) */
   while ((*ptr == ' ') && (*ptr != '\0') && (*ptr != '\n')) { ptr++; }
   if ( (*ptr == '\0') || (*ptr == '\n') ) {
      printf( "Error: Incomplete line in parameter file, no data start offset found\n" );
      return( 0 );
   }
   if ((*ptr < '0') || (*ptr > '9')) {
      printf( "Error: Non numeric data start offset, ptr at %s\n", ptr );
      return( 0 );
   }
   message_table.message_entry[message_table.num_entries].displaymsgdataoffset = atoi(ptr);

   /* find the next field, the message text */
   while ((*ptr != ' ') && (*ptr != '\0') && (*ptr != '\n')) { ptr++; }
   while ((*ptr == ' ') && (*ptr != '\0') && (*ptr != '\n')) { ptr++; }
   if ( (*ptr == '\0') || (*ptr == '\n') ) {
      printf( "Error: Incomplete line in parameter file, no message text\n" );
      return( 0 );
   }
   strncpy( message_table.message_entry[message_table.num_entries].msgscantext, ptr, MAX_SCAN_LEN );
   message_table.message_entry[message_table.num_entries].msgscantext[MAX_SCAN_LEN] = '\0';
   message_table.message_entry[message_table.num_entries].msglen = strlen(
            message_table.message_entry[message_table.num_entries].msgscantext );

   /* we have what we need, save and move on */
   message_table.num_entries++;
   return( 1 );
} /* store_line */


/* ---------------------------------------------------------------
 * line_function_request:
 *
 * This function comapres a line just read from stdin against the
 * entries in the memory table to see if the line read in matches
 * any of the rules we have recorded in the memory table.
 *
 * If a line matched the proc writes out the correct line for the
 * rule type.
 * --------------------------------------------------------------- */
void line_function_request( char *msgline ) {
   int i, line_len, work_line_len, foundmatch;
   char *ptr, *ptr2;

/* MID: 16Jan2004, added to support extracting the hostname value */
   int j;

   /* see if we can find a matching scan entry for the line */
   line_len = strlen(msgline);
   i = 0;
   foundmatch = -1;
   while ( (i < message_table.num_entries) && (foundmatch < 0) ) {
      /* Only scan lines that are large enough to contain the data being
	   * searched for. */
      if (line_len >= message_table.message_entry[i].msglen) {  /* it is big enough */
          ptr = msgline;                                                   /* ptr is start of line */
          /* we will use ptr2 to identify the point at which we stop scanning */
          ptr2 = ptr;  /* the start of the line */
          ptr2 = ptr2 + strlen(ptr);   /* now the end of the line */
          ptr2 = (ptr2 - message_table.message_entry[i].msglen) + 1; /* less the bytes we memcmp */
          while ( (ptr2 > ptr) && (foundmatch < 0) ) {
             if (memcmp(ptr,message_table.message_entry[i].msgscantext,message_table.message_entry[i].msglen) == 0) {
                 foundmatch = i;
             }
             else {
                ptr++;
             }
          }  /* while scanning string */
      } /* if line len big enough to check */
      i++;
   }  /* while searching for a match with the table */

   /* if a match was found (no longer -1) do the appropriate request */
   if (foundmatch >= 0) {
      /* MID: 16jan2004, are we required to extract a hostname field */
      if (usehostname_fieldnumber > 0) {
         ptr = msgline;                                                /* ptr is start of line */
         i = usehostname_fieldnumber - 1;                              /* number of spaces to skip */
	 while ((i > 0) && (*ptr != '\n') && (*ptr != '\0')) {
            if (*ptr == ' ') { i = i - 1; }
            ptr++;
         }
	 while (*ptr == ' ') { ptr++; }
	 ptr2 = ptr;
	 while ((*ptr2 != ' ') && (*ptr2 != '\n') && (*ptr2 != '\0')) { ptr2++; }
	 j = ptr2 - ptr;
	 if (j > MAX_HOSTNAME_LEN) { j = MAX_HOSTNAME_LEN; }
	 memcpy( reportinghost, ptr, j );
	 reportinghost[j] = '\0';
      }
      /* move the pointer back to the start of the line to be displayed */
      ptr = msgline;                                                   /* ptr is start of line */
      ptr = ptr + message_table.message_entry[foundmatch].displaymsgdataoffset; /* + text offset in line */
	  /* be nice, clean up any leading spaces in the remainder also */
	  while (*ptr == ' ') { ptr++; }
      /* the output we produce is based on the action the message table specified we take */
         /* MID: 16jan2004, added an extra trailing space%s for the hostname field */
      /* Raise an alert if A */
      if (message_table.message_entry[foundmatch].msgtype == 'A') {
          printf( "RAISE-ALERT %s \"%s\" %s\n", message_table.message_entry[foundmatch].alertkey, ptr, reportinghost );
      }
      /* Cancel an alert if X */
      else if (message_table.message_entry[foundmatch].msgtype == 'X') {
          printf( "CANCEL-ALERT %s \"%s\" %s\n", message_table.message_entry[foundmatch].alertkey, ptr, reportinghost );
      }
      /* User function 1-5 if 1-5 */
      else if ( (message_table.message_entry[foundmatch].msgtype >= '1') &&
                (message_table.message_entry[foundmatch].msgtype <= '5') ) {
          printf( "USERFUNC-%c %s \"%s\" %s\n",
                  message_table.message_entry[foundmatch].msgtype,
                  message_table.message_entry[foundmatch].alertkey, ptr, reportinghost );
      }
      /* else an error */
      else {
          /* an error slipped through our store_line checks, discard the line */
      }
	  /* It's been saving lots of messages before writing them, by flushing
	   * stdout after each alert written we ensure it is written in a timely
	   * fashion (rather than waiting until the buffer is full in an hour or so). */
	  i = fflush(stdout);
   } /* if a match was found */

   return;
} /* line_function_request */


/* ---------------------------------------------------------------
 * main:
 * Open and proces the config file, close it again.
 * Loop reading messages from stdin and parsing them, until the
 * pipe feeding us the messages closes.
 * --------------------------------------------------------------- */
int main(  int argc, char **argv ) {
   int parm_linecount, x;

   /* MID: 16Jan2004, added to support extracting the hostname value */
   usehostname_fieldnumber = 0; /* set the usehostname flag to 0, original behaviour */
   strcpy(reportinghost,"");   /* empty field, original behaviour */

   /* check we have a parameter filename */
   if (argc < 2) {
      printf( "Error: missing parameters !\n" );
      printf( "Syntax: tail -f <log-to-parse> | log_parser <config-file-name> [<logdataoffset>] | processing script\n" );
      exit( 1 );
   }

   /* Open the config file */
   if ( (infile = fopen( argv[1], "r" )) == NULL ) {
      snprintf( line_buffer, MAX_INPUT_LINE_LEN, "Error: Unable to open file %s\n", argv[1] );
      perror( line_buffer );
      exit( 1 );
   } /* failed */
   /* read the config input lines, expected to be...
    *    Action Type, Key, Offset, MsgText */
   message_table.num_entries = 0;  /* no entries yet */
   parm_linecount = 1;             /* used to help user with config file errors */
   while ( ( fgets(line_buffer, MAX_INPUT_LINE_LEN, infile) != NULL ) && (!(feof(infile))) ) {
      /* some lines have a new line at the end of the data that throws out our
       * scans, set to null if the last byte read was a new line */
      x = strlen(line_buffer) - 1;
	  if (line_buffer[x] == '\n') { line_buffer[x] = '\0'; }
      if (store_line( (char *)&line_buffer ) == 0) { /* error */
         fclose( infile );
         printf( "Error: config file syntax error (line %d) : %s\n", parm_linecount, line_buffer );
         exit( 1 );
      };
      parm_linecount++;
   } /* while lines are being read */
   /* Close the config file */
   fclose( infile );

   /* Read stdin until EOF now, processing each message */
   while ( ( fgets(line_buffer, MAX_INPUT_LINE_LEN, stdin) != NULL ) && (!(feof(stdin))) ) {
      /* new lines throw out our checks here also,replace ant trailing nl with
       * a null */
      x = strlen(line_buffer) - 1;
      if (line_buffer[x] == '\n') { line_buffer[x] = '\0'; }
      line_function_request((char *)&line_buffer);
   } /* end of input from stdin */

   /* so we are done */
   exit( 0 );
} /* end of main */
