/*
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
*/

/* 
 *  raise_alert.c :
 *
 *  This is provided so batch jobs can raise alerts for the
 *  alert collector process to manage. It is provided to allow
 *  an alerting facility for application that are not tightly 
 *  coupled to my automation and management suite of products.
 *
 * Note1: the unique_key is very important, the server uses it to
 * manage the alerts, and this program will use it in a harmless
 * message to delete a message this program raised as an alert. You
 * should keep it unique for each subsystem (ie: KEY_MEM, KEY_SWAP
 * so you can alert if mem/swap low and delete it when mem ok etc).
 * The unique key and source server are used as the key by the server
 * application so you could use KEY_MEM on all server and it would
 * still be unique to the alert collector process.
 * 
 * Note2: if you raise two alerts with the same key the second will
 * overwrite the first, so keys do need to be unique if you want
 * more than one alert from a server.
 *
 * Note3: all alerts from this utility are raised as persistent,
 * they will not be deleted when the program closes the socket. This
 * is different from my integrated applications (ie: job scheduler)
 * which raise non-persistent alerts which are deleted automatically
 * when the application (job scheduler) closes the socket to the alert
 * collector server. This is required as this program closes the socket
 * as soon as we post the alert but we don't want it to disapear.
 *
 *  syntax: raise_alert port ipaddr severity unique-key "message text"
 *
 * port is the port the server is listening on
 * ipaddr is the ipaddress of the server
 * severity may be one of 'critical', 'warning' or 'harmless' where..
 *    critical - a critical alert, all console apps should show these
 *    warning  - warning only, some console apps may not show these
 *    harmless - used to delete an alert previously raised.
 *    info - informational only
 *    logonly - only written to the central servers log file, never buffered as an alert
 *    clearall - deletes all alerts for the requesting host from the alert collector server,
 *               designed only for server shutdown events so a machine can run this from an
 *               rc script as it goes down.
 * unique-key must be a string that uniquely identifies the message (max 14 bytes)
 * "message text" is any message sting for the alert (max 100 bytes)
 *    
 *  examples:
 *  1: a script runs to check space in the /var filesystem and alerts if > 90%.
 *     ./raise_alert 9003 127.0.0.1 critical DISK_VAR "/var is over 90% full"
 *     someone cleans up some work files to free up disk space and the next
 *     time the script runs
 *     ./raise_alert 9003 127.0.0.1 harmless DISK_VAR "alert over: /var is now below 90% full"
 *     Note: this is an over simplification, you will need to record when you
 *     raise the alert, and when you clear it as you DO NOT want to send a
 *     harmless event every OK check, only when you know there is an alert to
 *     erase (or the server has a lot of overhead trying to find non-existent
 *     alerts, and if the server memtable is full it will even buffer it for
 *     later processing creating an even larger overhead).
 *   -- in a case like disk full I just raise the critical and never bother
 *   about the harmless, each new critical will replace the erlier until
 *   someone does something about the problem, after which it can be manually
 *   deleted through the console.
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include "system_type.h"

#include "../common/alertserver_api_def.h"
#include "../common/shared_utils.c"

/* *****************************************************************************
 * init_alert_request_buffer:
 * Initialise the alert message buffer, set it to default to a critical
 * message.
   ***************************************************************************** */
void init_alert_request_buffer( message_data_def * databuf ) {
   strncpy( databuf->msg_header.productid, "GENALERT", ASRV_PRODUCT_ID_LEN );
   strncpy( databuf->msg_header.version, application_version, ASRV_PRODUCT_VERSION_LEN );
   gethostname( (char *)&databuf->msg_header.hostname, MAX_ASRV_HOSTNAME_LEN );
   strncpy( databuf->msg_header.unique_key, "N/A", MAX_ASRV_KEY_LEN );
   strncpy( databuf->msg_header.response_port, "0", 6 );  /* not implemeted yet */
   databuf->msg_header.messagetype[0] = 'A';  /* alert */
   databuf->msg_header.messagetype[1] = 'C';  /* critical */
   databuf->msg_header.expect_reply = 'N';    /* this NEVER wants a reply */

   set_datestamp( (char *)&databuf->msg_header.source_date_time );
   databuf->datapersists = 'Y';    /* keep after we disconnectas we alsways disconnect  */
   zero_data_field( (char *)&databuf->datastr, MAX_ASRV_REMOTEHOST_DATA_LEN );
   zero_data_field( (char *)&databuf->optdata, MAX_ASRV_OPT_DATA );
   number_to_string_with_null( (char *)&databuf->datastrlen, 0, MAX_ASRV_LEN_STR+1 );
   pad_header_fields( (common_header_def *)databuf );
   databuf->NL = '\n';
} /*init_alert_request_buffer */

/* *****************************************************************************
 * connect_to_server:
 * Return 1 if OK, 0 if sockptr needed to be set to null
   ***************************************************************************** */
int connect_to_server(int portnum, char *ipaddr, int *sockptr) {
   int z;
   char   workbuf[4096];
   struct sockaddr_in adr_srvr; /* AF_INET */
   int    len_inet;           /* length */

   /* create server socket address */
   memset(&adr_srvr,0,sizeof adr_srvr);
   adr_srvr.sin_family = AF_INET;
   adr_srvr.sin_port = htons(portnum);
   adr_srvr.sin_addr.s_addr = inet_addr(ipaddr);
#ifdef SOLARIS
   /* Solaris does not have a INADDR_NONE defined
	* but returns -1 for a bad address */
   if (adr_srvr.sin_addr.s_addr == -1 )
#else
   if (adr_srvr.sin_addr.s_addr == INADDR_NONE )
#endif
   {
      printf( "Improperly formatted network address, %s\n", ipaddr );
      return( 0 );  /* failed */
   }
   len_inet = sizeof adr_srvr;

   /* create a tcpip socket to use */
   *sockptr = socket(PF_INET,SOCK_STREAM,0);
   if (*sockptr == -1) {
      printf( "%s: %s\n", strerror(errno), "Unable to create TCPIP socket" );
	  return( 0 ); /* failed */ 
   }

   /* connect to the server */
   z = connect(*sockptr,&adr_srvr,len_inet);
   if (z == -1) {
      snprintf( workbuf, 4095, "connect(2), host %s nothing listening on port %d", ipaddr, portnum );
      printf( "%s: %s\n", strerror(errno), workbuf );
	  return( 0 ); /* failed */
   }

   return( 1 );  /* All OK */
} /* connect_to_server*/

/* *****************************************************************************
 * disconnect_from_server:
 * Use *sockptr to pass the socket to close.
   ***************************************************************************** */
void disconnect_from_server( int sockptr ) {
   /* close the socket and exit */
   close(sockptr);
   putchar('\n');
   return;
} /* disconnect_from_server */


/* *****************************************************************************
 * main:
   ***************************************************************************** */
int main(int argc, char **argv) {
   int z, stop_now, command_result;
   int socketid; /* socket */
   char *zptr;
   char *srvr_addr = NULL;
   int  srvr_port;
   message_data_def alert_message;
   char workbuffer[100];

   /* check all required arguments are present, we need
	* port numer, ipaddress, severity, key, message text */
   if (argc < 5) {
      printf( "Required arguments are missing !\n" );
      printf( "Need port-num ip-address severity key message-text\n" );
	  printf( "Please read the manual !\n" );
      exit(1);
   }

   /* initialise the alert buffer */
   init_alert_request_buffer( &alert_message );

   /* get the port number */
   srvr_port = atoi(argv[1]);
   if (srvr_port == 0) {
      printf( "Illegal port number: %s\n", argv[1] );
      exit( 1 );
   }
   /* get the ip address */
   srvr_addr = argv[2];
   /* get the severity */
   if (memcmp( argv[3], "critical", 8) == 0) {
       alert_message.msg_header.messagetype[0] = 'A'; /* AC is alert critical */
       alert_message.msg_header.messagetype[1] = 'C';
   }
   else if (memcmp( argv[3], "warning", 7) == 0) {
       alert_message.msg_header.messagetype[0] = 'A'; /* AW is alert warning */
       alert_message.msg_header.messagetype[1] = 'W';
   }
   else if (memcmp( argv[3], "harmless", 8) == 0) {
       alert_message.msg_header.messagetype[0] = 'A'; /* AK is alert kill */
       alert_message.msg_header.messagetype[1] = 'K';
   }
   else if (memcmp( argv[3], "info", 4) == 0) {
       alert_message.msg_header.messagetype[0] = 'I'; /* IO is informational */
       alert_message.msg_header.messagetype[1] = 'O';
   }
   else if (memcmp( argv[3], "logonly", 7) == 0) {
       alert_message.msg_header.messagetype[0] = 'L'; /* LO is log only */
       alert_message.msg_header.messagetype[1] = 'O';
   }
   else if (memcmp( argv[3], "clearall", 8) == 0) {
       alert_message.msg_header.messagetype[0] = 'K'; /* KH is clear all for host */
       alert_message.msg_header.messagetype[1] = 'H';
   }
   /* else not supported */
   else
   {
      printf( "Invalid severity agrument !\n" );
	  printf( "Please read the manual !\n" );
      exit(1);
   }
   /* now get the unique key */
   strncpy( alert_message.msg_header.unique_key, argv[4], MAX_ASRV_KEY_LEN );
   alert_message.msg_header.unique_key[MAX_ASRV_KEY_LEN] = '\0'; /* ensure terminated with a null */
   /* now get the alert text */
   strncpy( alert_message.datastr, argv[5], MAX_ASRV_REMOTEHOST_DATA_LEN );
   alert_message.datastr[MAX_ASRV_REMOTEHOST_DATA_LEN] = '\0'; /* ensure terminated with a null */
   number_to_string_with_null( (char *)&alert_message.datastrlen, strlen(alert_message.datastr), MAX_ASRV_LEN_STR+1 );
   alert_message.datastrlen[MAX_ASRV_LEN_STR] = '\0'; /* wasn't null terminated, is now */

   /* ensure the alert key fields are space padded */
   while (strlen(alert_message.msg_header.unique_key) < MAX_ASRV_KEY_LEN) {
      strcat(alert_message.msg_header.unique_key, " ");
   }
   while (strlen(alert_message.msg_header.hostname) < MAX_ASRV_HOSTNAME_LEN) {
      strcat(alert_message.msg_header.hostname, " ");
   }

   /* Connect to the alert server */
   if ((command_result = connect_to_server( srvr_port, srvr_addr, &socketid )) == 0) {
      printf( "Unable to connect to server.\n" );
      exit( 1 );
   }

   /* ----------------------------------------------
	* all parms retrieved and message built, send it.
    * ---------------------------------------------- */
   z = write( socketid, &alert_message, sizeof(message_data_def) );
   if ( z != sizeof(message_data_def) ) {
      printf( "Unable to write alert buffer to %s\n", srvr_addr );
   }

   /* ----------------------------------------------
	* We have finished.
    * close the socket and exit
    * ---------------------------------------------- */
   /* close the socket and exit */
   disconnect_from_server( socketid );
   return(0);
} /* main */
